# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmLocalExecutionSpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cpu_count': 'float',
        'gpu_count': 'int',
        'gpu_type': 'str',
        'hostname': 'str',
        'os': 'str',
        'output_volume_id': 'int',
        'python_executable': 'str',
        'python_version': 'str',
        'start_command': 'str',
        'vessl_version': 'str'
    }

    attribute_map = {
        'cpu_count': 'cpu_count',
        'gpu_count': 'gpu_count',
        'gpu_type': 'gpu_type',
        'hostname': 'hostname',
        'os': 'os',
        'output_volume_id': 'output_volume_id',
        'python_executable': 'python_executable',
        'python_version': 'python_version',
        'start_command': 'start_command',
        'vessl_version': 'vessl_version'
    }

    def __init__(self, cpu_count=None, gpu_count=None, gpu_type=None, hostname=None, os=None, output_volume_id=None, python_executable=None, python_version=None, start_command=None, vessl_version=None, local_vars_configuration=None):  # noqa: E501
        """OrmLocalExecutionSpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cpu_count = None
        self._gpu_count = None
        self._gpu_type = None
        self._hostname = None
        self._os = None
        self._output_volume_id = None
        self._python_executable = None
        self._python_version = None
        self._start_command = None
        self._vessl_version = None
        self.discriminator = None

        self.cpu_count = cpu_count
        self.gpu_count = gpu_count
        self.gpu_type = gpu_type
        self.hostname = hostname
        self.os = os
        self.output_volume_id = output_volume_id
        self.python_executable = python_executable
        self.python_version = python_version
        self.start_command = start_command
        self.vessl_version = vessl_version

    @property
    def cpu_count(self):
        """Gets the cpu_count of this OrmLocalExecutionSpec.  # noqa: E501


        :return: The cpu_count of this OrmLocalExecutionSpec.  # noqa: E501
        :rtype: float
        """
        return self._cpu_count

    @cpu_count.setter
    def cpu_count(self, cpu_count):
        """Sets the cpu_count of this OrmLocalExecutionSpec.


        :param cpu_count: The cpu_count of this OrmLocalExecutionSpec.  # noqa: E501
        :type cpu_count: float
        """

        self._cpu_count = cpu_count

    @property
    def gpu_count(self):
        """Gets the gpu_count of this OrmLocalExecutionSpec.  # noqa: E501


        :return: The gpu_count of this OrmLocalExecutionSpec.  # noqa: E501
        :rtype: int
        """
        return self._gpu_count

    @gpu_count.setter
    def gpu_count(self, gpu_count):
        """Sets the gpu_count of this OrmLocalExecutionSpec.


        :param gpu_count: The gpu_count of this OrmLocalExecutionSpec.  # noqa: E501
        :type gpu_count: int
        """

        self._gpu_count = gpu_count

    @property
    def gpu_type(self):
        """Gets the gpu_type of this OrmLocalExecutionSpec.  # noqa: E501


        :return: The gpu_type of this OrmLocalExecutionSpec.  # noqa: E501
        :rtype: str
        """
        return self._gpu_type

    @gpu_type.setter
    def gpu_type(self, gpu_type):
        """Sets the gpu_type of this OrmLocalExecutionSpec.


        :param gpu_type: The gpu_type of this OrmLocalExecutionSpec.  # noqa: E501
        :type gpu_type: str
        """

        self._gpu_type = gpu_type

    @property
    def hostname(self):
        """Gets the hostname of this OrmLocalExecutionSpec.  # noqa: E501


        :return: The hostname of this OrmLocalExecutionSpec.  # noqa: E501
        :rtype: str
        """
        return self._hostname

    @hostname.setter
    def hostname(self, hostname):
        """Sets the hostname of this OrmLocalExecutionSpec.


        :param hostname: The hostname of this OrmLocalExecutionSpec.  # noqa: E501
        :type hostname: str
        """

        self._hostname = hostname

    @property
    def os(self):
        """Gets the os of this OrmLocalExecutionSpec.  # noqa: E501


        :return: The os of this OrmLocalExecutionSpec.  # noqa: E501
        :rtype: str
        """
        return self._os

    @os.setter
    def os(self, os):
        """Sets the os of this OrmLocalExecutionSpec.


        :param os: The os of this OrmLocalExecutionSpec.  # noqa: E501
        :type os: str
        """

        self._os = os

    @property
    def output_volume_id(self):
        """Gets the output_volume_id of this OrmLocalExecutionSpec.  # noqa: E501


        :return: The output_volume_id of this OrmLocalExecutionSpec.  # noqa: E501
        :rtype: int
        """
        return self._output_volume_id

    @output_volume_id.setter
    def output_volume_id(self, output_volume_id):
        """Sets the output_volume_id of this OrmLocalExecutionSpec.


        :param output_volume_id: The output_volume_id of this OrmLocalExecutionSpec.  # noqa: E501
        :type output_volume_id: int
        """
        if self.local_vars_configuration.client_side_validation and output_volume_id is None:  # noqa: E501
            raise ValueError("Invalid value for `output_volume_id`, must not be `None`")  # noqa: E501

        self._output_volume_id = output_volume_id

    @property
    def python_executable(self):
        """Gets the python_executable of this OrmLocalExecutionSpec.  # noqa: E501


        :return: The python_executable of this OrmLocalExecutionSpec.  # noqa: E501
        :rtype: str
        """
        return self._python_executable

    @python_executable.setter
    def python_executable(self, python_executable):
        """Sets the python_executable of this OrmLocalExecutionSpec.


        :param python_executable: The python_executable of this OrmLocalExecutionSpec.  # noqa: E501
        :type python_executable: str
        """

        self._python_executable = python_executable

    @property
    def python_version(self):
        """Gets the python_version of this OrmLocalExecutionSpec.  # noqa: E501


        :return: The python_version of this OrmLocalExecutionSpec.  # noqa: E501
        :rtype: str
        """
        return self._python_version

    @python_version.setter
    def python_version(self, python_version):
        """Sets the python_version of this OrmLocalExecutionSpec.


        :param python_version: The python_version of this OrmLocalExecutionSpec.  # noqa: E501
        :type python_version: str
        """

        self._python_version = python_version

    @property
    def start_command(self):
        """Gets the start_command of this OrmLocalExecutionSpec.  # noqa: E501


        :return: The start_command of this OrmLocalExecutionSpec.  # noqa: E501
        :rtype: str
        """
        return self._start_command

    @start_command.setter
    def start_command(self, start_command):
        """Sets the start_command of this OrmLocalExecutionSpec.


        :param start_command: The start_command of this OrmLocalExecutionSpec.  # noqa: E501
        :type start_command: str
        """

        self._start_command = start_command

    @property
    def vessl_version(self):
        """Gets the vessl_version of this OrmLocalExecutionSpec.  # noqa: E501


        :return: The vessl_version of this OrmLocalExecutionSpec.  # noqa: E501
        :rtype: str
        """
        return self._vessl_version

    @vessl_version.setter
    def vessl_version(self, vessl_version):
        """Sets the vessl_version of this OrmLocalExecutionSpec.


        :param vessl_version: The vessl_version of this OrmLocalExecutionSpec.  # noqa: E501
        :type vessl_version: str
        """

        self._vessl_version = vessl_version

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmLocalExecutionSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmLocalExecutionSpec):
            return True

        return self.to_dict() != other.to_dict()



from test.conftest import USE_MOCK

import pytest

import vessl


@pytest.mark.skipif(USE_MOCK, reason="Does not run if mocking is used.")
class TestKernelImage:
    @pytest.mark.order(index=1)
    def test_list_kernel_images(self):
        images = vessl.list_kernel_images()
        pytest.image = images[0]

    def test_read_kernel_image(self):
        vessl.read_kernel_image(pytest.image.id)



import os
import sys
from typing import List

project_root = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
# this line should be on the top to run it on a local environment as 'python vessl/cli/_main.py'
sys.path.append(project_root)

import click
import sentry_sdk
from click.decorators import pass_context
from sentry_sdk.integrations.logging import ignore_logger

import vessl
from vessl._version import __VERSION__
from vessl.cli._base import VesslGroup
from vessl.cli._util import print_warning, prompt_choices
from vessl.cli.dataset import cli as dataset_cli
from vessl.cli.experiment import cli as experiment_cli
from vessl.cli.experiment import create as experiment_create
from vessl.cli.experiment import logs as experiment_logs
from vessl.cli.kernel_cluster import cli as kernel_cluster_cli
from vessl.cli.kernel_image import cli as kernel_image_cli
from vessl.cli.kernel_resource_spec import cli as kernel_resource_spec_cli
from vessl.cli.model import model_cli, model_repository_cli
from vessl.cli.organization import cli as organization_cli
from vessl.cli.project import cli as project_cli
from vessl.cli.run import cli as run_cli
from vessl.cli.serve import cli as serve_cli
from vessl.cli.ssh_key import cli as ssh_key_cli
from vessl.cli.sweep import cli as sweep_cli
from vessl.cli.volume import cli as volume_cli
from vessl.cli.workspace import cli as workspace_cli
from vessl.experiment import list_github_code_refs
from vessl.run import run_from_yaml, wrap_str, create_run
from vessl.util.common import safe_cast
from vessl.util.config import DEFAULT_CONFIG_PATH, VesslConfigLoader
from vessl.util.constant import (
    ENABLE_SENTRY,
    EXPERIMENT_WORKING_DIR,
    LANGCHAIN,
    NANO_GPT,
    SEGMENT_ANYTHING,
    SENTRY_DSN,
    STABLE_DIFFUSION,
    THIN_PLATE_SPLINE_MOTION,
    VESSL_ENV,
)
from vessl.util.exception import (
    InvalidOrganizationError,
    InvalidProjectError,
    InvalidTokenError,
    VesslApiException,
)
from vessl.util.git_remote import clone_codes, clone_codes_v2

# Configure Sentry in production
if VESSL_ENV == "prod" and ENABLE_SENTRY:
    sentry_sdk.init(
        SENTRY_DSN,
        traces_sample_rate=1.0,
        ignore_errors=[VesslApiException],
    )
    sentry_sdk.set_tag("cli_version", __VERSION__)
    ignore_logger("vessl.util.logger")
else:
    print_warning(f"Sentry is disabled - VESSL_ENV: {VESSL_ENV}, ENABLE_SENTRY: {ENABLE_SENTRY}")


def prompt_organizations() -> str:
    organizations = vessl.list_organizations()
    organization_count = len(organizations)
    if organization_count == 1:
        return organizations[0].name

    new_organization_string = "Create new organization..."
    choices = [(x.name, i) for i, x in enumerate(organizations)] + [
        (new_organization_string, organization_count)
    ]
    choice = prompt_choices("Default organization", choices)

    if choice == organization_count:
        organization_name = click.prompt("Organization name", type=click.STRING)
        vessl.create_organization(organization_name)
    else:
        organization_name = organizations[choice].name

    return organization_name


@click.command(cls=VesslGroup)
@click.version_option()
@pass_context
def cli(ctx: click.Context):
    vessl.EXEC_MODE = "CLI"
    ctx.ensure_object(dict)


@cli.group(cls=VesslGroup, invoke_without_command=True)
@click.pass_context
@click.option("-t", "--access-token", type=click.STRING)
@click.option("-o", "--organization", type=click.STRING)
@click.option("-p", "--project", type=click.STRING)
@click.option("-f", "--credentials-file", type=click.STRING)
@click.option("--renew-token", is_flag=True)
@click.option("--reset", is_flag=True)
def configure(
    ctx,
    access_token: str,
    organization: str,
    project: str,
    credentials_file: str,
    renew_token: bool,
    reset: bool,
):
    if ctx.invoked_subcommand:
        return

    if reset:
        vessl.vessl_api.config_loader = VesslConfigLoader()
        vessl.vessl_api.config_loader.reset()

    try:
        vessl.configure_access_token(
            access_token=access_token,
            credentials_file=credentials_file,
            force_update=renew_token,
        )
    except InvalidTokenError:
        vessl.configure_access_token(force_update=True)

    try:
        vessl.configure_organization(
            organization_name=organization,
            credentials_file=credentials_file,
        )
    except InvalidOrganizationError:
        organization_name = prompt_organizations()
        vessl.configure_organization(organization_name)

    try:
        vessl.configure_project(
            project_name=project,
            credentials_file=credentials_file,
        )
    except InvalidProjectError:
        projects = vessl.list_projects()
        if len(projects) == 0:
            project_name = None
        elif len(projects) == 1:
            project_name = projects[0].name
        else:
            new_project_string = "Create new project..."
            project_count = len(projects)
            choices = [(x.name, i) for i, x in enumerate(projects)] + [
                (new_project_string, project_count)
            ]
            choice = prompt_choices("Default project", choices)

        if choice == project_count:
            project_name = click.prompt("Project name", type=click.STRING)
            vessl.create_project(project_name)
        else:
            project_name = projects[choice].name

        if project_name is not None:
            vessl.configure_project(project_name)

    print(f"Welcome, {vessl.vessl_api.user.display_name}!")


@cli.group(cls=VesslGroup, invoke_without_command=True)
def whoami():
    config = VesslConfigLoader()
    user = None
    if config.access_token:
        vessl.vessl_api.api_client.set_default_header(
            "Authorization", f"Token {config.access_token}"
        )

        try:
            user = vessl.vessl_api.get_my_user_info_api()
        except VesslApiException:
            pass

    organization_name = config.default_organization
    project_name = config.default_project

    if user is None or organization_name is None:
        print("Please run `vessl configure` first.")
        return

    print(
        f"""Username: {user.username}
Email: {user.email}
Default organization: {organization_name}
Default project: {project_name or 'N/A'}

(The default organization and project can be updated with `vessl configure --reset`.)"""
    )


@configure.vessl_command()
@click.argument("organization", type=click.STRING, required=False)
def organization(organization: str):
    if organization is None:
        organization = prompt_organizations()
    vessl.configure_organization(organization)
    print(f"Saved to {DEFAULT_CONFIG_PATH}.")


@configure.vessl_command()
@click.argument("project", type=click.STRING, required=False)
def project(project: str):
    vessl.vessl_api.set_organization()

    if project is None:
        projects = vessl.list_projects()
        if len(projects) == 0:
            return

        project = prompt_choices("Default project", [x.name for x in projects])
    vessl.configure_project(project)
    print(f"Saved to {DEFAULT_CONFIG_PATH}.")


@configure.command()
def list():
    config = VesslConfigLoader()

    username = ""
    email = ""
    organization = config.default_organization or ""
    project = config.default_project or ""

    if config.access_token:
        vessl.vessl_api.api_client.set_default_header(
            "Authorization", f"Token {config.access_token}"
        )

        try:
            user = vessl.vessl_api.get_my_user_info_api()
            username = user.username
            email = user.email
        except VesslApiException as e:
            pass

    print(
        f"Username: {username}\n"
        f"Email: {email}\n"
        f"Organization: {organization}\n"
        f"Project: {project}"
    )


@cli.vessl_run_command()
@click.pass_context
@click.argument("command", nargs=-1)
@click.option(
    "-f",
    "--file",
    type=click.STRING,
    help="yaml file for run definition.",
)
def run(ctx, command: List[str], file: str):
    if len(command) == 0 and file == None:
        print("Missing argument")
        return
    # configure
    ctx.params = {}
    args = []
    ctx.args = configure.parse_args(ctx, args)
    ctx.forward(configure)

    if file:
        if not os.path.exists(file):
            print(wrap_str(f" Yaml does not exist! Check yaml path {file}.", "red"))
            return
        with open(file, "r") as yaml_file:
            created_run = run_from_yaml(yaml_file=yaml_file, yaml_body="", yaml_file_name=file)
    elif len(command) > 0:
        args = []
        args.extend(["--command", " ".join(command)])
        args.extend(["--working-dir", f"{EXPERIMENT_WORKING_DIR}local/"])
        args.extend(["--upload-local-file", f".:{EXPERIMENT_WORKING_DIR}local/"])

        ctx.params = {}
        ctx.args = experiment_create.parse_args(ctx, args)
        ctx.forward(experiment_create)

        experiment_number = ctx.obj.get("experiment_number")
        ctx.params = {}
        ctx.args = experiment_logs.parse_args(ctx, [str(experiment_number), "--follow"])
        ctx.forward(experiment_logs)
    else:
        pass


@cli.vessl_run_command()
@click.pass_context
def hello(ctx):
    ctx.params = {}
    args = []
    ctx.args = configure.parse_args(ctx, args)
    ctx.forward(configure)

    choices = [
        ("[nanoGPT] Train on Shakespeare's works.", 1),
        ("[Stable Diffusion] Run an interactive stable diffusion demo.", 2),
        ("[Segment Anything] Try the FAIR's Segment Anything demo.", 3),
        ("[LangChain] Generate a personal assistant given the context.", 4),
        ("[Thin-Plate Spline Motion Model] Inference images on the model to animate them.", 5),
    ]
    choice = prompt_choices("Choose an example run", choices)
    if choice == 1:
        created_run = create_run(
            yaml_file=None, yaml_body=NANO_GPT, yaml_file_name="nanogpt-yaml.yaml"
        )
    elif choice == 2:
        created_run = create_run(
            yaml_file=None, yaml_body=STABLE_DIFFUSION, yaml_file_name="stable-diffusion.yaml"
        )
    elif choice == 3:
        created_run = create_run(
            yaml_file=None, yaml_body=SEGMENT_ANYTHING, yaml_file_name="segment-anything.yaml"
        )
    elif choice == 4:
        created_run = create_run(
            yaml_file=None, yaml_body=LANGCHAIN, yaml_file_name="langchain.yaml"
        )
    elif choice == 5:
        created_run = create_run(
            yaml_file=None,
            yaml_body=THIN_PLATE_SPLINE_MOTION,
            yaml_file_name="thin-plate-spline-motion.yaml",
        )


@cli.vessl_command(hidden=True)
def download_code():
    """
    This command is not for users and supposed to run on initContainers of experiment, pipeline, and workspace Pods.
    """
    workload_id = safe_cast(os.environ.get("VESSL_WORKLOAD_ID", None), int)
    if workload_id is None:
        print("Failed to download source codes: no workload id.")
        return

    resp = list_github_code_refs(workload_id)
    if resp.scheme == "v2":
        github_code_refs = resp.results_v2
        if not github_code_refs:
            print("No source code to download.")
            return
        clone_codes_v2(github_code_refs)
    else:
        github_code_refs = resp.results
        if not github_code_refs:
            print("No source code to download.")
            return
        clone_codes(github_code_refs)


cli.add_command(dataset_cli)
cli.add_command(experiment_cli)
cli.add_command(kernel_cluster_cli)
cli.add_command(kernel_image_cli)
cli.add_command(kernel_resource_spec_cli)
cli.add_command(model_cli)
cli.add_command(model_repository_cli)
cli.add_command(organization_cli)
cli.add_command(project_cli)
cli.add_command(run_cli)
cli.add_command(serve_cli)
cli.add_command(ssh_key_cli)
cli.add_command(sweep_cli)
cli.add_command(volume_cli)
cli.add_command(workspace_cli)


if __name__ == "__main__":
    cli()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmOnPremiseVolumeConfigFlexVolume(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'credentials_id': 'int',
        'domain': 'str',
        'network_path': 'str',
        'options': 'list[OrmOnPremiseVolumeConfigFlexVolumeOptionsInner]',
        'password': 'str',
        'secret_name': 'str',
        'username': 'str'
    }

    attribute_map = {
        'credentials_id': 'credentials_id',
        'domain': 'domain',
        'network_path': 'network_path',
        'options': 'options',
        'password': 'password',
        'secret_name': 'secret_name',
        'username': 'username'
    }

    def __init__(self, credentials_id=None, domain=None, network_path=None, options=None, password=None, secret_name=None, username=None, local_vars_configuration=None):  # noqa: E501
        """OrmOnPremiseVolumeConfigFlexVolume - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._credentials_id = None
        self._domain = None
        self._network_path = None
        self._options = None
        self._password = None
        self._secret_name = None
        self._username = None
        self.discriminator = None

        if credentials_id is not None:
            self.credentials_id = credentials_id
        if domain is not None:
            self.domain = domain
        self.network_path = network_path
        if options is not None:
            self.options = options
        if password is not None:
            self.password = password
        if secret_name is not None:
            self.secret_name = secret_name
        if username is not None:
            self.username = username

    @property
    def credentials_id(self):
        """Gets the credentials_id of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501


        :return: The credentials_id of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :rtype: int
        """
        return self._credentials_id

    @credentials_id.setter
    def credentials_id(self, credentials_id):
        """Sets the credentials_id of this OrmOnPremiseVolumeConfigFlexVolume.


        :param credentials_id: The credentials_id of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :type credentials_id: int
        """

        self._credentials_id = credentials_id

    @property
    def domain(self):
        """Gets the domain of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501


        :return: The domain of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :rtype: str
        """
        return self._domain

    @domain.setter
    def domain(self, domain):
        """Sets the domain of this OrmOnPremiseVolumeConfigFlexVolume.


        :param domain: The domain of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :type domain: str
        """

        self._domain = domain

    @property
    def network_path(self):
        """Gets the network_path of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501


        :return: The network_path of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :rtype: str
        """
        return self._network_path

    @network_path.setter
    def network_path(self, network_path):
        """Sets the network_path of this OrmOnPremiseVolumeConfigFlexVolume.


        :param network_path: The network_path of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :type network_path: str
        """
        if self.local_vars_configuration.client_side_validation and network_path is None:  # noqa: E501
            raise ValueError("Invalid value for `network_path`, must not be `None`")  # noqa: E501

        self._network_path = network_path

    @property
    def options(self):
        """Gets the options of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501


        :return: The options of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :rtype: list[OrmOnPremiseVolumeConfigFlexVolumeOptionsInner]
        """
        return self._options

    @options.setter
    def options(self, options):
        """Sets the options of this OrmOnPremiseVolumeConfigFlexVolume.


        :param options: The options of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :type options: list[OrmOnPremiseVolumeConfigFlexVolumeOptionsInner]
        """

        self._options = options

    @property
    def password(self):
        """Gets the password of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501


        :return: The password of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :rtype: str
        """
        return self._password

    @password.setter
    def password(self, password):
        """Sets the password of this OrmOnPremiseVolumeConfigFlexVolume.


        :param password: The password of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :type password: str
        """

        self._password = password

    @property
    def secret_name(self):
        """Gets the secret_name of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501


        :return: The secret_name of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :rtype: str
        """
        return self._secret_name

    @secret_name.setter
    def secret_name(self, secret_name):
        """Sets the secret_name of this OrmOnPremiseVolumeConfigFlexVolume.


        :param secret_name: The secret_name of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :type secret_name: str
        """

        self._secret_name = secret_name

    @property
    def username(self):
        """Gets the username of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501


        :return: The username of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :rtype: str
        """
        return self._username

    @username.setter
    def username(self, username):
        """Sets the username of this OrmOnPremiseVolumeConfigFlexVolume.


        :param username: The username of this OrmOnPremiseVolumeConfigFlexVolume.  # noqa: E501
        :type username: str
        """

        self._username = username

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmOnPremiseVolumeConfigFlexVolume):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmOnPremiseVolumeConfigFlexVolume):
            return True

        return self.to_dict() != other.to_dict()



import os
import pathlib
import re
from typing import List, Union

from vessl.util import logger


class InvalidPatternException(Exception):
    pass


# Regex that matches all parent directories ("ab/cd/ef/", for example)
REGEX_ALL_DIRS_PREFIX = r"([^/]+/)*"


def _segment_to_regex(segment: str) -> str:
    """
    Given a pattern for a path segment (i.e. a name of a directory or a file), returns a regex
    pattern that matches the segment exactly.

    It supports `*` (matches zero or more characters) and `?` (matches exactly one character).
    These wildcards do not match directory separator (`/`).
    """
    translate_map = {i: "\\" + chr(i) for i in b"()[]{}+-|^$\\.&~# \t\n\r\v\f"}
    # translate takes keys as integers only
    translate_map[b"?"[0]] = "[^/]"
    translate_map[b"*"[0]] = "[^/]*"
    return segment.translate(translate_map)


class Pattern(object):
    """
    Pattern parses and holds a single line in vesslignore file.

    Args:
        pattern (str): A pattern string in .vesslignore file.

    Attributes:
        _regex (re.Pattern): Parsed pattern in regex.
            A path can be checked by _regex.match() (match from the beginning).
    """

    _regex: re.Pattern

    def __init__(self, pattern: str):
        self._parse(pattern)
        logger.debug(f"Parsed pattern {pattern} into regex {self._regex.pattern}")

    def __str__(self) -> str:
        return f"<Pattern: regex {self._regex.pattern}>"

    def __repr__(self) -> str:
        return f"<Pattern: regex {self._regex.pattern}>"

    def _parse(self, pattern: str):
        if self._try_parse_simple(pattern):
            return

        if self._try_parse_all_dirs_prefixed(pattern):
            return

        if self._try_parse_relative(pattern):
            return

        raise InvalidPatternException(pattern)

    def _try_parse_simple(self, pattern: str) -> bool:
        """
        Try to parse a pattern string as a simple field. Both files and directories can match
        against this pattern.

        It supports `*` (matches zero or more characters) and `?` (matches exactly one character).
        These wildcards do not match against directory separator (`/`).

        Examples include:
        - `db.sqlite3` (all files or directories with this name in all subdirectories will match)
        - `*.pyc`
        - `.*.sw?` (example for Vim swap-files)
        - `__pycache__` (note that directories also match this pattern)

        Args:
            pattern (str): A pattern string in .vesslignore file.

        Returns:
            True if parse has succeeded.
        """

        PATTERN_REGEX_SIMPLE = r"[^/]+"
        if re.fullmatch(PATTERN_REGEX_SIMPLE, pattern) is None:
            return False

        regex = REGEX_ALL_DIRS_PREFIX + _segment_to_regex(pattern)

        regex += r"(/|$)"  # match file ($) or directory (/)

        self._regex = re.compile(regex)
        return True

    def _try_parse_all_dirs_prefixed(self, pattern: str) -> bool:
        """
        Try to parse a pattern string with preceding `**/`. Both files and directories can match
        against this pattern; a trailing directory separator (`/`) will force a directory match.

        The remaining part of the pattern should not contain neither of `*`, `?`, nor `//`, and
        denotes a pattern that matches against any file or directory in the tree.

        Examples include:
        - `**/bin/Debug/` (matches all `Debug` directories that are under `bin`)
        - `**/nbproject/private` (matches all files or directories that are named `private`
          and are under `nbproject`)

        Args:
            pattern (str): A pattern string in .vesslignore file.

        Returns:
            True if parse has succeeded.
        """
        PATTERN_REGEX_ALL_DIRS_PREFIX = r"\*\*/[^*?]+"
        if re.fullmatch(PATTERN_REGEX_ALL_DIRS_PREFIX, pattern) is None:
            return False
        if "//" in pattern or "**" in pattern[1:]:
            return False

        regex = REGEX_ALL_DIRS_PREFIX
        pattern_body = pattern[3:]
        if "*" in pattern_body or "?" in pattern_body:
            return False

        trailing_slash = False
        if pattern_body.endswith("/"):
            trailing_slash = True
            pattern_body = pattern_body[:-1]
            if not pattern_body:
                # "**//" case
                return False

        path_segments = pattern_body.split("/")
        regex += r"/".join(map(_segment_to_regex, path_segments))

        # At this point, pattern `**/a/b` may match against path `a/bc`;
        # we should add a delimiter. Use trailing_slash to determine which delimiter to use.
        regex += r"/" if trailing_slash else r"(/|$)"

        self._regex = re.compile(regex)
        return True

    def _try_parse_relative(self, pattern: str) -> bool:
        """
        Try to parse a pattern string that matches relatively to current directory.
        Both files and directories can match against this pattern; a trailing directory separator
        (`/`) will force a directory match.

        It supports `*` (matches zero or more characters) and `?` (matches exactly one character).
        These wildcards do not match against directory separator.

        The pattern string should not contain `**` or `//`. Leading directory separator is ignored.

        Note:
            The pattern should contain at least one directory separator character to match this case;
            otherwise, it would be considered as a 'simple' pattern (like `*.so`).

        Examples include:
        - `.venv/` (matches `.venv` directory at the same level as this .vesslignore file)
        - `*.egg-info/`
        - `` (may match file or directory)

        Args:
            pattern (str): A pattern string in .vesslignore file.

        Returns:
            True if parse has succeeded.
        """
        if "**" in pattern or "//" in pattern:
            return False

        if "/" not in pattern:
            return False

        trailing_slash = pattern.endswith("/")
        pattern = pattern.strip("/")  # strip both leading and trailing slash
        if not pattern:
            # "/" case
            return False

        path_segments = pattern.split("/")
        regex = r"/".join(map(_segment_to_regex, path_segments))

        # At this point, pattern `a/b` may match against path `a/bc`;
        # we should add a delimiter. Use trailing_slash to determine which delimiter to use.
        regex += r"/" if trailing_slash else r"(/|$)"

        self._regex = re.compile(regex)
        return True

    def matches(self, path: str) -> bool:
        return self._regex.fullmatch(path) is not None



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class V1MountClaim(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cifs': 'V1CIFSMount',
        'csi': 'V1CSIMount',
        'dataset': 'V1DatasetMount',
        'driver_type': 'str',
        'googledisk': 'V1GoogleDiskMount',
        'hostpath': 'V1HostPathMount',
        'nfs': 'V1NFSMount'
    }

    attribute_map = {
        'cifs': 'cifs',
        'csi': 'csi',
        'dataset': 'dataset',
        'driver_type': 'driver_type',
        'googledisk': 'googledisk',
        'hostpath': 'hostpath',
        'nfs': 'nfs'
    }

    def __init__(self, cifs=None, csi=None, dataset=None, driver_type=None, googledisk=None, hostpath=None, nfs=None, local_vars_configuration=None):  # noqa: E501
        """V1MountClaim - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cifs = None
        self._csi = None
        self._dataset = None
        self._driver_type = None
        self._googledisk = None
        self._hostpath = None
        self._nfs = None
        self.discriminator = None

        if cifs is not None:
            self.cifs = cifs
        if csi is not None:
            self.csi = csi
        if dataset is not None:
            self.dataset = dataset
        self.driver_type = driver_type
        if googledisk is not None:
            self.googledisk = googledisk
        if hostpath is not None:
            self.hostpath = hostpath
        if nfs is not None:
            self.nfs = nfs

    @property
    def cifs(self):
        """Gets the cifs of this V1MountClaim.  # noqa: E501


        :return: The cifs of this V1MountClaim.  # noqa: E501
        :rtype: V1CIFSMount
        """
        return self._cifs

    @cifs.setter
    def cifs(self, cifs):
        """Sets the cifs of this V1MountClaim.


        :param cifs: The cifs of this V1MountClaim.  # noqa: E501
        :type cifs: V1CIFSMount
        """

        self._cifs = cifs

    @property
    def csi(self):
        """Gets the csi of this V1MountClaim.  # noqa: E501


        :return: The csi of this V1MountClaim.  # noqa: E501
        :rtype: V1CSIMount
        """
        return self._csi

    @csi.setter
    def csi(self, csi):
        """Sets the csi of this V1MountClaim.


        :param csi: The csi of this V1MountClaim.  # noqa: E501
        :type csi: V1CSIMount
        """

        self._csi = csi

    @property
    def dataset(self):
        """Gets the dataset of this V1MountClaim.  # noqa: E501


        :return: The dataset of this V1MountClaim.  # noqa: E501
        :rtype: V1DatasetMount
        """
        return self._dataset

    @dataset.setter
    def dataset(self, dataset):
        """Sets the dataset of this V1MountClaim.


        :param dataset: The dataset of this V1MountClaim.  # noqa: E501
        :type dataset: V1DatasetMount
        """

        self._dataset = dataset

    @property
    def driver_type(self):
        """Gets the driver_type of this V1MountClaim.  # noqa: E501


        :return: The driver_type of this V1MountClaim.  # noqa: E501
        :rtype: str
        """
        return self._driver_type

    @driver_type.setter
    def driver_type(self, driver_type):
        """Sets the driver_type of this V1MountClaim.


        :param driver_type: The driver_type of this V1MountClaim.  # noqa: E501
        :type driver_type: str
        """
        if self.local_vars_configuration.client_side_validation and driver_type is None:  # noqa: E501
            raise ValueError("Invalid value for `driver_type`, must not be `None`")  # noqa: E501
        allowed_values = ["hostpath", "nfs", "cifs", "googledisk", "csi", "dataset"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and driver_type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `driver_type` ({0}), must be one of {1}"  # noqa: E501
                .format(driver_type, allowed_values)
            )

        self._driver_type = driver_type

    @property
    def googledisk(self):
        """Gets the googledisk of this V1MountClaim.  # noqa: E501


        :return: The googledisk of this V1MountClaim.  # noqa: E501
        :rtype: V1GoogleDiskMount
        """
        return self._googledisk

    @googledisk.setter
    def googledisk(self, googledisk):
        """Sets the googledisk of this V1MountClaim.


        :param googledisk: The googledisk of this V1MountClaim.  # noqa: E501
        :type googledisk: V1GoogleDiskMount
        """

        self._googledisk = googledisk

    @property
    def hostpath(self):
        """Gets the hostpath of this V1MountClaim.  # noqa: E501


        :return: The hostpath of this V1MountClaim.  # noqa: E501
        :rtype: V1HostPathMount
        """
        return self._hostpath

    @hostpath.setter
    def hostpath(self, hostpath):
        """Sets the hostpath of this V1MountClaim.


        :param hostpath: The hostpath of this V1MountClaim.  # noqa: E501
        :type hostpath: V1HostPathMount
        """

        self._hostpath = hostpath

    @property
    def nfs(self):
        """Gets the nfs of this V1MountClaim.  # noqa: E501


        :return: The nfs of this V1MountClaim.  # noqa: E501
        :rtype: V1NFSMount
        """
        return self._nfs

    @nfs.setter
    def nfs(self, nfs):
        """Sets the nfs of this V1MountClaim.


        :param nfs: The nfs of this V1MountClaim.  # noqa: E501
        :type nfs: V1NFSMount
        """

        self._nfs = nfs

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1MountClaim):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1MountClaim):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmAutoscalerConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'average_utilization': 'int',
        'max_replicas': 'int',
        'min_replicas': 'int',
        'resource_target': 'str'
    }

    attribute_map = {
        'average_utilization': 'average_utilization',
        'max_replicas': 'max_replicas',
        'min_replicas': 'min_replicas',
        'resource_target': 'resource_target'
    }

    def __init__(self, average_utilization=None, max_replicas=None, min_replicas=None, resource_target=None, local_vars_configuration=None):  # noqa: E501
        """OrmAutoscalerConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._average_utilization = None
        self._max_replicas = None
        self._min_replicas = None
        self._resource_target = None
        self.discriminator = None

        if average_utilization is not None:
            self.average_utilization = average_utilization
        if max_replicas is not None:
            self.max_replicas = max_replicas
        if min_replicas is not None:
            self.min_replicas = min_replicas
        if resource_target is not None:
            self.resource_target = resource_target

    @property
    def average_utilization(self):
        """Gets the average_utilization of this OrmAutoscalerConfig.  # noqa: E501


        :return: The average_utilization of this OrmAutoscalerConfig.  # noqa: E501
        :rtype: int
        """
        return self._average_utilization

    @average_utilization.setter
    def average_utilization(self, average_utilization):
        """Sets the average_utilization of this OrmAutoscalerConfig.


        :param average_utilization: The average_utilization of this OrmAutoscalerConfig.  # noqa: E501
        :type average_utilization: int
        """

        self._average_utilization = average_utilization

    @property
    def max_replicas(self):
        """Gets the max_replicas of this OrmAutoscalerConfig.  # noqa: E501


        :return: The max_replicas of this OrmAutoscalerConfig.  # noqa: E501
        :rtype: int
        """
        return self._max_replicas

    @max_replicas.setter
    def max_replicas(self, max_replicas):
        """Sets the max_replicas of this OrmAutoscalerConfig.


        :param max_replicas: The max_replicas of this OrmAutoscalerConfig.  # noqa: E501
        :type max_replicas: int
        """

        self._max_replicas = max_replicas

    @property
    def min_replicas(self):
        """Gets the min_replicas of this OrmAutoscalerConfig.  # noqa: E501


        :return: The min_replicas of this OrmAutoscalerConfig.  # noqa: E501
        :rtype: int
        """
        return self._min_replicas

    @min_replicas.setter
    def min_replicas(self, min_replicas):
        """Sets the min_replicas of this OrmAutoscalerConfig.


        :param min_replicas: The min_replicas of this OrmAutoscalerConfig.  # noqa: E501
        :type min_replicas: int
        """

        self._min_replicas = min_replicas

    @property
    def resource_target(self):
        """Gets the resource_target of this OrmAutoscalerConfig.  # noqa: E501


        :return: The resource_target of this OrmAutoscalerConfig.  # noqa: E501
        :rtype: str
        """
        return self._resource_target

    @resource_target.setter
    def resource_target(self, resource_target):
        """Sets the resource_target of this OrmAutoscalerConfig.


        :param resource_target: The resource_target of this OrmAutoscalerConfig.  # noqa: E501
        :type resource_target: str
        """

        self._resource_target = resource_target

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmAutoscalerConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmAutoscalerConfig):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseModelServiceRolloutInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_by': 'ResponseUser',
        'created_dt': 'datetime',
        'current_step': 'int',
        'desired_endpoint': 'str',
        'desired_traffic_split': 'list[OrmModelServiceGatewayTrafficSplitEntry]',
        'message': 'str',
        'model_service_name': 'str',
        'number': 'int',
        'related_revisions': 'list[ResponseModelServiceRolloutRelatedRevisionInfo]',
        'rollout_spec': 'OrmModelServiceRolloutSpec',
        'status': 'str',
        'step_statuses': 'dict[str, OrmModelServiceRolloutStepStatus]'
    }

    attribute_map = {
        'created_by': 'created_by',
        'created_dt': 'created_dt',
        'current_step': 'current_step',
        'desired_endpoint': 'desired_endpoint',
        'desired_traffic_split': 'desired_traffic_split',
        'message': 'message',
        'model_service_name': 'model_service_name',
        'number': 'number',
        'related_revisions': 'related_revisions',
        'rollout_spec': 'rollout_spec',
        'status': 'status',
        'step_statuses': 'step_statuses'
    }

    def __init__(self, created_by=None, created_dt=None, current_step=None, desired_endpoint=None, desired_traffic_split=None, message=None, model_service_name=None, number=None, related_revisions=None, rollout_spec=None, status=None, step_statuses=None, local_vars_configuration=None):  # noqa: E501
        """ResponseModelServiceRolloutInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_by = None
        self._created_dt = None
        self._current_step = None
        self._desired_endpoint = None
        self._desired_traffic_split = None
        self._message = None
        self._model_service_name = None
        self._number = None
        self._related_revisions = None
        self._rollout_spec = None
        self._status = None
        self._step_statuses = None
        self.discriminator = None

        self.created_by = created_by
        self.created_dt = created_dt
        self.current_step = current_step
        if desired_endpoint is not None:
            self.desired_endpoint = desired_endpoint
        if desired_traffic_split is not None:
            self.desired_traffic_split = desired_traffic_split
        self.message = message
        self.model_service_name = model_service_name
        self.number = number
        if related_revisions is not None:
            self.related_revisions = related_revisions
        self.rollout_spec = rollout_spec
        self.status = status
        self.step_statuses = step_statuses

    @property
    def created_by(self):
        """Gets the created_by of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The created_by of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: ResponseUser
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this ResponseModelServiceRolloutInfo.


        :param created_by: The created_by of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type created_by: ResponseUser
        """
        if self.local_vars_configuration.client_side_validation and created_by is None:  # noqa: E501
            raise ValueError("Invalid value for `created_by`, must not be `None`")  # noqa: E501

        self._created_by = created_by

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The created_dt of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseModelServiceRolloutInfo.


        :param created_dt: The created_dt of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type created_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `created_dt`, must not be `None`")  # noqa: E501

        self._created_dt = created_dt

    @property
    def current_step(self):
        """Gets the current_step of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The current_step of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: int
        """
        return self._current_step

    @current_step.setter
    def current_step(self, current_step):
        """Sets the current_step of this ResponseModelServiceRolloutInfo.


        :param current_step: The current_step of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type current_step: int
        """
        if self.local_vars_configuration.client_side_validation and current_step is None:  # noqa: E501
            raise ValueError("Invalid value for `current_step`, must not be `None`")  # noqa: E501

        self._current_step = current_step

    @property
    def desired_endpoint(self):
        """Gets the desired_endpoint of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The desired_endpoint of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: str
        """
        return self._desired_endpoint

    @desired_endpoint.setter
    def desired_endpoint(self, desired_endpoint):
        """Sets the desired_endpoint of this ResponseModelServiceRolloutInfo.


        :param desired_endpoint: The desired_endpoint of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type desired_endpoint: str
        """

        self._desired_endpoint = desired_endpoint

    @property
    def desired_traffic_split(self):
        """Gets the desired_traffic_split of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The desired_traffic_split of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: list[OrmModelServiceGatewayTrafficSplitEntry]
        """
        return self._desired_traffic_split

    @desired_traffic_split.setter
    def desired_traffic_split(self, desired_traffic_split):
        """Sets the desired_traffic_split of this ResponseModelServiceRolloutInfo.


        :param desired_traffic_split: The desired_traffic_split of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type desired_traffic_split: list[OrmModelServiceGatewayTrafficSplitEntry]
        """

        self._desired_traffic_split = desired_traffic_split

    @property
    def message(self):
        """Gets the message of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The message of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this ResponseModelServiceRolloutInfo.


        :param message: The message of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type message: str
        """
        if self.local_vars_configuration.client_side_validation and message is None:  # noqa: E501
            raise ValueError("Invalid value for `message`, must not be `None`")  # noqa: E501

        self._message = message

    @property
    def model_service_name(self):
        """Gets the model_service_name of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The model_service_name of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: str
        """
        return self._model_service_name

    @model_service_name.setter
    def model_service_name(self, model_service_name):
        """Sets the model_service_name of this ResponseModelServiceRolloutInfo.


        :param model_service_name: The model_service_name of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type model_service_name: str
        """
        if self.local_vars_configuration.client_side_validation and model_service_name is None:  # noqa: E501
            raise ValueError("Invalid value for `model_service_name`, must not be `None`")  # noqa: E501

        self._model_service_name = model_service_name

    @property
    def number(self):
        """Gets the number of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The number of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: int
        """
        return self._number

    @number.setter
    def number(self, number):
        """Sets the number of this ResponseModelServiceRolloutInfo.


        :param number: The number of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type number: int
        """
        if self.local_vars_configuration.client_side_validation and number is None:  # noqa: E501
            raise ValueError("Invalid value for `number`, must not be `None`")  # noqa: E501

        self._number = number

    @property
    def related_revisions(self):
        """Gets the related_revisions of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The related_revisions of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: list[ResponseModelServiceRolloutRelatedRevisionInfo]
        """
        return self._related_revisions

    @related_revisions.setter
    def related_revisions(self, related_revisions):
        """Sets the related_revisions of this ResponseModelServiceRolloutInfo.


        :param related_revisions: The related_revisions of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type related_revisions: list[ResponseModelServiceRolloutRelatedRevisionInfo]
        """

        self._related_revisions = related_revisions

    @property
    def rollout_spec(self):
        """Gets the rollout_spec of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The rollout_spec of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: OrmModelServiceRolloutSpec
        """
        return self._rollout_spec

    @rollout_spec.setter
    def rollout_spec(self, rollout_spec):
        """Sets the rollout_spec of this ResponseModelServiceRolloutInfo.


        :param rollout_spec: The rollout_spec of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type rollout_spec: OrmModelServiceRolloutSpec
        """
        if self.local_vars_configuration.client_side_validation and rollout_spec is None:  # noqa: E501
            raise ValueError("Invalid value for `rollout_spec`, must not be `None`")  # noqa: E501

        self._rollout_spec = rollout_spec

    @property
    def status(self):
        """Gets the status of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The status of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ResponseModelServiceRolloutInfo.


        :param status: The status of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def step_statuses(self):
        """Gets the step_statuses of this ResponseModelServiceRolloutInfo.  # noqa: E501


        :return: The step_statuses of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :rtype: dict[str, OrmModelServiceRolloutStepStatus]
        """
        return self._step_statuses

    @step_statuses.setter
    def step_statuses(self, step_statuses):
        """Sets the step_statuses of this ResponseModelServiceRolloutInfo.


        :param step_statuses: The step_statuses of this ResponseModelServiceRolloutInfo.  # noqa: E501
        :type step_statuses: dict[str, OrmModelServiceRolloutStepStatus]
        """
        if self.local_vars_configuration.client_side_validation and step_statuses is None:  # noqa: E501
            raise ValueError("Invalid value for `step_statuses`, must not be `None`")  # noqa: E501

        self._step_statuses = step_statuses

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseModelServiceRolloutInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseModelServiceRolloutInfo):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class V1RunSpecForm(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'description': 'str',
        'env': 'dict[str, V1EnvForm]',
        'export': 'dict[str, V1ExportForm]',
        'image': 'V1ImageForm',
        '_import': 'dict[str, V1ImportForm]',
        'interactive': 'V1InteractiveForm',
        'mount': 'dict[str, V1MountForm]',
        'name': 'str',
        'ports': 'list[V1PortForm]',
        'resources': 'V1ResourcesForm',
        'run': 'list[V1CommandForm]',
        'service_account_name': 'str',
        'tags': 'list[str]',
        'termination_protection': 'bool'
    }

    attribute_map = {
        'description': 'description',
        'env': 'env',
        'export': 'export',
        'image': 'image',
        '_import': 'import',
        'interactive': 'interactive',
        'mount': 'mount',
        'name': 'name',
        'ports': 'ports',
        'resources': 'resources',
        'run': 'run',
        'service_account_name': 'service_account_name',
        'tags': 'tags',
        'termination_protection': 'termination_protection'
    }

    def __init__(self, description=None, env=None, export=None, image=None, _import=None, interactive=None, mount=None, name=None, ports=None, resources=None, run=None, service_account_name=None, tags=None, termination_protection=None, local_vars_configuration=None):  # noqa: E501
        """V1RunSpecForm - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._description = None
        self._env = None
        self._export = None
        self._image = None
        self.__import = None
        self._interactive = None
        self._mount = None
        self._name = None
        self._ports = None
        self._resources = None
        self._run = None
        self._service_account_name = None
        self._tags = None
        self._termination_protection = None
        self.discriminator = None

        self.description = description
        if env is not None:
            self.env = env
        if export is not None:
            self.export = export
        self.image = image
        if _import is not None:
            self._import = _import
        if interactive is not None:
            self.interactive = interactive
        if mount is not None:
            self.mount = mount
        self.name = name
        if ports is not None:
            self.ports = ports
        self.resources = resources
        if run is not None:
            self.run = run
        self.service_account_name = service_account_name
        if tags is not None:
            self.tags = tags
        self.termination_protection = termination_protection

    @property
    def description(self):
        """Gets the description of this V1RunSpecForm.  # noqa: E501


        :return: The description of this V1RunSpecForm.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this V1RunSpecForm.


        :param description: The description of this V1RunSpecForm.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def env(self):
        """Gets the env of this V1RunSpecForm.  # noqa: E501


        :return: The env of this V1RunSpecForm.  # noqa: E501
        :rtype: dict[str, V1EnvForm]
        """
        return self._env

    @env.setter
    def env(self, env):
        """Sets the env of this V1RunSpecForm.


        :param env: The env of this V1RunSpecForm.  # noqa: E501
        :type env: dict[str, V1EnvForm]
        """

        self._env = env

    @property
    def export(self):
        """Gets the export of this V1RunSpecForm.  # noqa: E501


        :return: The export of this V1RunSpecForm.  # noqa: E501
        :rtype: dict[str, V1ExportForm]
        """
        return self._export

    @export.setter
    def export(self, export):
        """Sets the export of this V1RunSpecForm.


        :param export: The export of this V1RunSpecForm.  # noqa: E501
        :type export: dict[str, V1ExportForm]
        """

        self._export = export

    @property
    def image(self):
        """Gets the image of this V1RunSpecForm.  # noqa: E501


        :return: The image of this V1RunSpecForm.  # noqa: E501
        :rtype: V1ImageForm
        """
        return self._image

    @image.setter
    def image(self, image):
        """Sets the image of this V1RunSpecForm.


        :param image: The image of this V1RunSpecForm.  # noqa: E501
        :type image: V1ImageForm
        """
        if self.local_vars_configuration.client_side_validation and image is None:  # noqa: E501
            raise ValueError("Invalid value for `image`, must not be `None`")  # noqa: E501

        self._image = image

    @property
    def _import(self):
        """Gets the _import of this V1RunSpecForm.  # noqa: E501


        :return: The _import of this V1RunSpecForm.  # noqa: E501
        :rtype: dict[str, V1ImportForm]
        """
        return self.__import

    @_import.setter
    def _import(self, _import):
        """Sets the _import of this V1RunSpecForm.


        :param _import: The _import of this V1RunSpecForm.  # noqa: E501
        :type _import: dict[str, V1ImportForm]
        """

        self.__import = _import

    @property
    def interactive(self):
        """Gets the interactive of this V1RunSpecForm.  # noqa: E501


        :return: The interactive of this V1RunSpecForm.  # noqa: E501
        :rtype: V1InteractiveForm
        """
        return self._interactive

    @interactive.setter
    def interactive(self, interactive):
        """Sets the interactive of this V1RunSpecForm.


        :param interactive: The interactive of this V1RunSpecForm.  # noqa: E501
        :type interactive: V1InteractiveForm
        """

        self._interactive = interactive

    @property
    def mount(self):
        """Gets the mount of this V1RunSpecForm.  # noqa: E501


        :return: The mount of this V1RunSpecForm.  # noqa: E501
        :rtype: dict[str, V1MountForm]
        """
        return self._mount

    @mount.setter
    def mount(self, mount):
        """Sets the mount of this V1RunSpecForm.


        :param mount: The mount of this V1RunSpecForm.  # noqa: E501
        :type mount: dict[str, V1MountForm]
        """

        self._mount = mount

    @property
    def name(self):
        """Gets the name of this V1RunSpecForm.  # noqa: E501


        :return: The name of this V1RunSpecForm.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this V1RunSpecForm.


        :param name: The name of this V1RunSpecForm.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def ports(self):
        """Gets the ports of this V1RunSpecForm.  # noqa: E501


        :return: The ports of this V1RunSpecForm.  # noqa: E501
        :rtype: list[V1PortForm]
        """
        return self._ports

    @ports.setter
    def ports(self, ports):
        """Sets the ports of this V1RunSpecForm.


        :param ports: The ports of this V1RunSpecForm.  # noqa: E501
        :type ports: list[V1PortForm]
        """

        self._ports = ports

    @property
    def resources(self):
        """Gets the resources of this V1RunSpecForm.  # noqa: E501


        :return: The resources of this V1RunSpecForm.  # noqa: E501
        :rtype: V1ResourcesForm
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this V1RunSpecForm.


        :param resources: The resources of this V1RunSpecForm.  # noqa: E501
        :type resources: V1ResourcesForm
        """
        if self.local_vars_configuration.client_side_validation and resources is None:  # noqa: E501
            raise ValueError("Invalid value for `resources`, must not be `None`")  # noqa: E501

        self._resources = resources

    @property
    def run(self):
        """Gets the run of this V1RunSpecForm.  # noqa: E501


        :return: The run of this V1RunSpecForm.  # noqa: E501
        :rtype: list[V1CommandForm]
        """
        return self._run

    @run.setter
    def run(self, run):
        """Sets the run of this V1RunSpecForm.


        :param run: The run of this V1RunSpecForm.  # noqa: E501
        :type run: list[V1CommandForm]
        """

        self._run = run

    @property
    def service_account_name(self):
        """Gets the service_account_name of this V1RunSpecForm.  # noqa: E501


        :return: The service_account_name of this V1RunSpecForm.  # noqa: E501
        :rtype: str
        """
        return self._service_account_name

    @service_account_name.setter
    def service_account_name(self, service_account_name):
        """Sets the service_account_name of this V1RunSpecForm.


        :param service_account_name: The service_account_name of this V1RunSpecForm.  # noqa: E501
        :type service_account_name: str
        """

        self._service_account_name = service_account_name

    @property
    def tags(self):
        """Gets the tags of this V1RunSpecForm.  # noqa: E501


        :return: The tags of this V1RunSpecForm.  # noqa: E501
        :rtype: list[str]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this V1RunSpecForm.


        :param tags: The tags of this V1RunSpecForm.  # noqa: E501
        :type tags: list[str]
        """

        self._tags = tags

    @property
    def termination_protection(self):
        """Gets the termination_protection of this V1RunSpecForm.  # noqa: E501


        :return: The termination_protection of this V1RunSpecForm.  # noqa: E501
        :rtype: bool
        """
        return self._termination_protection

    @termination_protection.setter
    def termination_protection(self, termination_protection):
        """Sets the termination_protection of this V1RunSpecForm.


        :param termination_protection: The termination_protection of this V1RunSpecForm.  # noqa: E501
        :type termination_protection: bool
        """

        self._termination_protection = termination_protection

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1RunSpecForm):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1RunSpecForm):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class V1Claim(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'export': 'V1ExportClaim',
        'export_to': 'V1ExportableReference',
        '_import': 'V1ImportClaim',
        'import_source': 'V1ImportableReference',
        'mount': 'V1MountClaim',
        'volume_capacity': 'str'
    }

    attribute_map = {
        'export': 'export',
        'export_to': 'export_to',
        '_import': 'import',
        'import_source': 'import_source',
        'mount': 'mount',
        'volume_capacity': 'volume_capacity'
    }

    def __init__(self, export=None, export_to=None, _import=None, import_source=None, mount=None, volume_capacity=None, local_vars_configuration=None):  # noqa: E501
        """V1Claim - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._export = None
        self._export_to = None
        self.__import = None
        self._import_source = None
        self._mount = None
        self._volume_capacity = None
        self.discriminator = None

        if export is not None:
            self.export = export
        if export_to is not None:
            self.export_to = export_to
        if _import is not None:
            self._import = _import
        if import_source is not None:
            self.import_source = import_source
        if mount is not None:
            self.mount = mount
        self.volume_capacity = volume_capacity

    @property
    def export(self):
        """Gets the export of this V1Claim.  # noqa: E501


        :return: The export of this V1Claim.  # noqa: E501
        :rtype: V1ExportClaim
        """
        return self._export

    @export.setter
    def export(self, export):
        """Sets the export of this V1Claim.


        :param export: The export of this V1Claim.  # noqa: E501
        :type export: V1ExportClaim
        """

        self._export = export

    @property
    def export_to(self):
        """Gets the export_to of this V1Claim.  # noqa: E501


        :return: The export_to of this V1Claim.  # noqa: E501
        :rtype: V1ExportableReference
        """
        return self._export_to

    @export_to.setter
    def export_to(self, export_to):
        """Sets the export_to of this V1Claim.


        :param export_to: The export_to of this V1Claim.  # noqa: E501
        :type export_to: V1ExportableReference
        """

        self._export_to = export_to

    @property
    def _import(self):
        """Gets the _import of this V1Claim.  # noqa: E501


        :return: The _import of this V1Claim.  # noqa: E501
        :rtype: V1ImportClaim
        """
        return self.__import

    @_import.setter
    def _import(self, _import):
        """Sets the _import of this V1Claim.


        :param _import: The _import of this V1Claim.  # noqa: E501
        :type _import: V1ImportClaim
        """

        self.__import = _import

    @property
    def import_source(self):
        """Gets the import_source of this V1Claim.  # noqa: E501


        :return: The import_source of this V1Claim.  # noqa: E501
        :rtype: V1ImportableReference
        """
        return self._import_source

    @import_source.setter
    def import_source(self, import_source):
        """Sets the import_source of this V1Claim.


        :param import_source: The import_source of this V1Claim.  # noqa: E501
        :type import_source: V1ImportableReference
        """

        self._import_source = import_source

    @property
    def mount(self):
        """Gets the mount of this V1Claim.  # noqa: E501


        :return: The mount of this V1Claim.  # noqa: E501
        :rtype: V1MountClaim
        """
        return self._mount

    @mount.setter
    def mount(self, mount):
        """Sets the mount of this V1Claim.


        :param mount: The mount of this V1Claim.  # noqa: E501
        :type mount: V1MountClaim
        """

        self._mount = mount

    @property
    def volume_capacity(self):
        """Gets the volume_capacity of this V1Claim.  # noqa: E501


        :return: The volume_capacity of this V1Claim.  # noqa: E501
        :rtype: str
        """
        return self._volume_capacity

    @volume_capacity.setter
    def volume_capacity(self, volume_capacity):
        """Sets the volume_capacity of this V1Claim.


        :param volume_capacity: The volume_capacity of this V1Claim.  # noqa: E501
        :type volume_capacity: str
        """

        self._volume_capacity = volume_capacity

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1Claim):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1Claim):
            return True

        return self.to_dict() != other.to_dict()



from typing import List

from openapi_client import (
    ModelServiceGatewayUpdateAPIInput,
    ModelserviceModelServiceListResponse,
    ModelserviceModelServiceRevisionListResponse,
)
from openapi_client import OrmModelServiceGatewayTrafficSplitEntry as TrafficSplitEntry
from openapi_client import (
    ResponseModelServiceGatewayInfo,
    ResponseModelServiceInfo,
    ResponseModelServiceRevision,
    ResponseSimpleModelServiceRevision,
    ServingGatewayYamlImportAPIInput,
    ServingRevisionYamlImportAPIInput,
)
from vessl import vessl_api


def list_servings(organization: str) -> ModelserviceModelServiceListResponse:
    """Get a list of all servings in an organization
    
    Args:
        organization(str): The name of the organization.

    Example:
        ```python
        vessl.list_servings(organization="my-org")
        ```
    """
    return vessl_api.model_service_list_api(organization_name=organization)


def create_revision_from_yaml(
    organization: str, serving_name: str, yaml_body: str
) -> ResponseModelServiceRevision:
    """Create a new revision of serving from a YAML file.

    Args:
        organization(str): The name of the organization.
        serving_name(str): The name of the serving.
        yaml_body(str): The YAML body of the serving.
            It is not deserialized YAML, but a whole yaml string.
    
    Example:
        ```python
        vessl.create_revision_from_yaml(
            organization="my-org",
            serving_name="my-serving",
            yaml_body=yaml_body)
        ```
    """
    payload = ServingRevisionYamlImportAPIInput(yaml_body)

    return vessl_api.serving_revision_yaml_import_api(
        organization_name=organization,
        model_service_name=serving_name,
        serving_revision_yaml_import_api_input=payload,
    )


def read_revision(
    organization: str, serving_name: str, revision_number: int
) -> ResponseModelServiceRevision:
    """Get a serving revision from a serving name and revision number.

    Args:
        organization(str): The name of the organization.
        serving_name(str): The name of the serving.
        revision_number(int): The revision number of the serving.
    
    Example:
        ```python
        vessl.read_revision(
            organization="my-org",
            serving_name="my-serving",
            revision_number=1)
        ```
    """
    return vessl_api.model_service_revision_read_api(
        organization_name=organization,
        model_service_name=serving_name,
        revision_number=revision_number,
    )


def list_revisions(
    organization: str, serving_name: str
) -> List[ResponseSimpleModelServiceRevision]:
    """Get a list of all revisions of a serving.

    Args:
        organization(str): The name of the organization.
        serving_name(str): The name of the serving.

    Examples:
        ```python
        vessl.list_revisions(
            organization="my-org",
            serving_name="my-serving")
        ```
    """
    resp: ModelserviceModelServiceRevisionListResponse = vessl_api.model_service_revision_list_api(
        organization_name=organization,
        model_service_name=serving_name,
    )
    return resp.results


def read_gateway(organization: str, serving_name: str) -> ResponseModelServiceGatewayInfo:
    """Get the gateway of a serving.

    Args:
        organization(str): The name of the organization.
        serving_name(str): The name of the serving.
    
    Examples:
        ```python
        vessl.read_gateway(
            organization="my-org",
            serving_name="my-serving")
        ```
    """
    model_service: ResponseModelServiceInfo = vessl_api.model_service_read_api(
        model_service_name=serving_name,
        organization_name=organization,
    )
    return model_service.gateway_config


def update_gateway(
    organization: str, serving_name: str, gateway: ModelServiceGatewayUpdateAPIInput
) -> ResponseModelServiceGatewayInfo:
    """Update the gateway of a serving.

    Args:
        organization(str): The name of the organization.
        serving_name(str): The name of the serving.
        gateway(ModelServiceGatewayUpdateAPIInput): The gateway of the serving.
    
    Examples:
        ```python
        from openapi_client import ModelServiceGatewayUpdateAPIInput
        from openapi_client import OrmModelServiceGatewayTrafficSplitEntry 

        gateway = ModelServiceGatewayUpdateAPIInput(
            enabled=True,
            endpoint="my-endpoint",
            traffic_split=[
                OrmModelServiceGatewayTrafficSplitEntry(
                    revision_number=1,
                    port=2222,
                    traffic_weight=100,
                )
            ],
        )

        vessl.update_gateway(
            organization="my-org",
            serving_name="my-serving",
            gateway=gateway)
        ```
    """
    return vessl_api.model_service_gateway_update_api(
        model_service_name=serving_name,
        organization_name=organization,
        model_service_gateway_update_api_input=gateway,
    )


def _get_updated_traffic_split_rule(
    rules_current: List[TrafficSplitEntry], revision_number: int, weight: int, port: int
) -> List[TrafficSplitEntry]:
    """
    Combines the previous traffic split rule with new rule.
    When filling the remaining weight, this function uses the one with higher revision number.

    For example, with the current rule of:
    - revision #2 (port 2222) 70%
    - revision #3 (port 3333) 30%

    with a call to this function with:
    - revision #4 (port 4444) 50%

    yields a new rule of:
    - revision #4 (port 4444) 50%
    - revision #3 (port 3333) 30%
    - revision #2 (port 2222) 20%

    Revision #3 takes priority over #2, because it has the higher number (3 > 2).
    """
    # Sort from latest revision (with highest number) to oldest
    rules_current = sorted(rules_current, key=lambda x: x.revision_number, reverse=True)

    rules_new: List[TrafficSplitEntry] = [
        TrafficSplitEntry(revision_number=revision_number, port=port, traffic_weight=weight)
    ]

    weight_remaining = 100 - weight

    # Iterate through current traffic rules and add them if possible
    for rule in rules_current:
        if weight_remaining <= 0:
            break
        new_weight = min(weight_remaining, rule.traffic_weight)
        rules_new.append(
            TrafficSplitEntry(
                revision_number=rule.revision_number, port=rule.port, traffic_weight=new_weight
            )
        )
        weight_remaining -= new_weight
        if weight_remaining <= 0:
            break

    if weight_remaining > 0:
        # This can happen if rules_current's weight do not sum up to 100
        # (this is possible for disabled gateways).
        # Handle this case safely by delegating all remaining weights to our target rule.
        rules_new[0].traffic_weight += weight_remaining

    return rules_new


def update_gateway_for_revision(
    organization: str,
    serving_name: str,
    revision_number: int,
    port: int,
    weight: int,
) -> ResponseModelServiceGatewayInfo:
    """ Update the current gateway of a serving for a specific revision.

    Args:
        organization(str): The name of the organization.
        serving_name(str): The name of the serving.
        revision_number(int): The revision number of the serving.
        port(int): The port of the revision will use for gateway.
        weight(int): The weight of the traffic will be distributed to revision_number.
    
    Examples:
        ```python
        vessl.update_gateway_for_revision(
            organization="my-org",
            serving_name="my-serving",
            revision_number=1,
            port=2222,
            weight=100)
        ```
    """
    gateway_current = read_gateway(organization=organization, serving_name=serving_name)

    rules_new = _get_updated_traffic_split_rule(
        rules_current=gateway_current.rules or [],
        revision_number=revision_number,
        port=port,
        weight=weight,
    )

    gateway_updated = vessl_api.model_service_gateway_update_api(
        organization_name=organization,
        model_service_name=serving_name,
        model_service_gateway_update_api_input=ModelServiceGatewayUpdateAPIInput(
            enabled=True,
            endpoint=gateway_current.endpoint,
            ingress_class=gateway_current.ingress_class,
            annotations=gateway_current.annotations,
            traffic_split=rules_new,
        ),
    )
    return gateway_updated


def update_gateway_from_yaml(
    organization: str, serving_name: str, yaml_body: str
) -> ResponseModelServiceGatewayInfo:
    """ Update the gateway of a serving from a YAML file.
    
    Args:
        organization(str): The name of the organization.
        serving_name(str): The name of the serving.
        yaml_body(str): The YAML body of the serving.
            It is not deserialized YAML, but a whole yaml string
    
    Examples:
        ```python
        vessl.update_gateway_from_yaml(
            organization="my-org",
            serving_name="my-serving",
            yaml_body=yaml_body)
        ```
    """
    payload = ServingGatewayYamlImportAPIInput(yaml_body)

    return vessl_api.serving_gateway_yaml_import_api(
        organization_name=organization,
        model_service_name=serving_name,
        serving_gateway_yaml_import_api_input=payload,
    )



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmProjectEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'dashboards': 'list[OrmProjectDashboard]',
        'experiment_fields': 'list[OrmProjectExperimentField]',
        'experiments': 'list[OrmExperiment]',
        'organization': 'OrmOrganization',
        'persistent_artifacts': 'list[OrmArtifact]',
        'pipelines': 'list[OrmPipeline]',
        'policies': 'list[OrmAccessControlPolicy]',
        'primary_owner': 'OrmUser',
        'project_datasets': 'list[OrmProjectDataset]',
        'repositories': 'list[OrmProjectRepository]',
        'run_executions': 'list[OrmRunExecution]',
        'run_specs': 'list[OrmRunSpec]',
        'service_account_subjects': 'list[OrmSubject]',
        'tag_group': 'OrmTagGroup',
        'volume': 'OrmVolume'
    }

    attribute_map = {
        'dashboards': 'dashboards',
        'experiment_fields': 'experiment_fields',
        'experiments': 'experiments',
        'organization': 'organization',
        'persistent_artifacts': 'persistent_artifacts',
        'pipelines': 'pipelines',
        'policies': 'policies',
        'primary_owner': 'primary_owner',
        'project_datasets': 'project_datasets',
        'repositories': 'repositories',
        'run_executions': 'run_executions',
        'run_specs': 'run_specs',
        'service_account_subjects': 'service_account_subjects',
        'tag_group': 'tag_group',
        'volume': 'volume'
    }

    def __init__(self, dashboards=None, experiment_fields=None, experiments=None, organization=None, persistent_artifacts=None, pipelines=None, policies=None, primary_owner=None, project_datasets=None, repositories=None, run_executions=None, run_specs=None, service_account_subjects=None, tag_group=None, volume=None, local_vars_configuration=None):  # noqa: E501
        """OrmProjectEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._dashboards = None
        self._experiment_fields = None
        self._experiments = None
        self._organization = None
        self._persistent_artifacts = None
        self._pipelines = None
        self._policies = None
        self._primary_owner = None
        self._project_datasets = None
        self._repositories = None
        self._run_executions = None
        self._run_specs = None
        self._service_account_subjects = None
        self._tag_group = None
        self._volume = None
        self.discriminator = None

        if dashboards is not None:
            self.dashboards = dashboards
        if experiment_fields is not None:
            self.experiment_fields = experiment_fields
        if experiments is not None:
            self.experiments = experiments
        if organization is not None:
            self.organization = organization
        if persistent_artifacts is not None:
            self.persistent_artifacts = persistent_artifacts
        if pipelines is not None:
            self.pipelines = pipelines
        if policies is not None:
            self.policies = policies
        if primary_owner is not None:
            self.primary_owner = primary_owner
        if project_datasets is not None:
            self.project_datasets = project_datasets
        if repositories is not None:
            self.repositories = repositories
        if run_executions is not None:
            self.run_executions = run_executions
        if run_specs is not None:
            self.run_specs = run_specs
        if service_account_subjects is not None:
            self.service_account_subjects = service_account_subjects
        if tag_group is not None:
            self.tag_group = tag_group
        if volume is not None:
            self.volume = volume

    @property
    def dashboards(self):
        """Gets the dashboards of this OrmProjectEdges.  # noqa: E501


        :return: The dashboards of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmProjectDashboard]
        """
        return self._dashboards

    @dashboards.setter
    def dashboards(self, dashboards):
        """Sets the dashboards of this OrmProjectEdges.


        :param dashboards: The dashboards of this OrmProjectEdges.  # noqa: E501
        :type dashboards: list[OrmProjectDashboard]
        """

        self._dashboards = dashboards

    @property
    def experiment_fields(self):
        """Gets the experiment_fields of this OrmProjectEdges.  # noqa: E501


        :return: The experiment_fields of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmProjectExperimentField]
        """
        return self._experiment_fields

    @experiment_fields.setter
    def experiment_fields(self, experiment_fields):
        """Sets the experiment_fields of this OrmProjectEdges.


        :param experiment_fields: The experiment_fields of this OrmProjectEdges.  # noqa: E501
        :type experiment_fields: list[OrmProjectExperimentField]
        """

        self._experiment_fields = experiment_fields

    @property
    def experiments(self):
        """Gets the experiments of this OrmProjectEdges.  # noqa: E501


        :return: The experiments of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmExperiment]
        """
        return self._experiments

    @experiments.setter
    def experiments(self, experiments):
        """Sets the experiments of this OrmProjectEdges.


        :param experiments: The experiments of this OrmProjectEdges.  # noqa: E501
        :type experiments: list[OrmExperiment]
        """

        self._experiments = experiments

    @property
    def organization(self):
        """Gets the organization of this OrmProjectEdges.  # noqa: E501


        :return: The organization of this OrmProjectEdges.  # noqa: E501
        :rtype: OrmOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this OrmProjectEdges.


        :param organization: The organization of this OrmProjectEdges.  # noqa: E501
        :type organization: OrmOrganization
        """

        self._organization = organization

    @property
    def persistent_artifacts(self):
        """Gets the persistent_artifacts of this OrmProjectEdges.  # noqa: E501


        :return: The persistent_artifacts of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmArtifact]
        """
        return self._persistent_artifacts

    @persistent_artifacts.setter
    def persistent_artifacts(self, persistent_artifacts):
        """Sets the persistent_artifacts of this OrmProjectEdges.


        :param persistent_artifacts: The persistent_artifacts of this OrmProjectEdges.  # noqa: E501
        :type persistent_artifacts: list[OrmArtifact]
        """

        self._persistent_artifacts = persistent_artifacts

    @property
    def pipelines(self):
        """Gets the pipelines of this OrmProjectEdges.  # noqa: E501


        :return: The pipelines of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmPipeline]
        """
        return self._pipelines

    @pipelines.setter
    def pipelines(self, pipelines):
        """Sets the pipelines of this OrmProjectEdges.


        :param pipelines: The pipelines of this OrmProjectEdges.  # noqa: E501
        :type pipelines: list[OrmPipeline]
        """

        self._pipelines = pipelines

    @property
    def policies(self):
        """Gets the policies of this OrmProjectEdges.  # noqa: E501


        :return: The policies of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmAccessControlPolicy]
        """
        return self._policies

    @policies.setter
    def policies(self, policies):
        """Sets the policies of this OrmProjectEdges.


        :param policies: The policies of this OrmProjectEdges.  # noqa: E501
        :type policies: list[OrmAccessControlPolicy]
        """

        self._policies = policies

    @property
    def primary_owner(self):
        """Gets the primary_owner of this OrmProjectEdges.  # noqa: E501


        :return: The primary_owner of this OrmProjectEdges.  # noqa: E501
        :rtype: OrmUser
        """
        return self._primary_owner

    @primary_owner.setter
    def primary_owner(self, primary_owner):
        """Sets the primary_owner of this OrmProjectEdges.


        :param primary_owner: The primary_owner of this OrmProjectEdges.  # noqa: E501
        :type primary_owner: OrmUser
        """

        self._primary_owner = primary_owner

    @property
    def project_datasets(self):
        """Gets the project_datasets of this OrmProjectEdges.  # noqa: E501


        :return: The project_datasets of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmProjectDataset]
        """
        return self._project_datasets

    @project_datasets.setter
    def project_datasets(self, project_datasets):
        """Sets the project_datasets of this OrmProjectEdges.


        :param project_datasets: The project_datasets of this OrmProjectEdges.  # noqa: E501
        :type project_datasets: list[OrmProjectDataset]
        """

        self._project_datasets = project_datasets

    @property
    def repositories(self):
        """Gets the repositories of this OrmProjectEdges.  # noqa: E501


        :return: The repositories of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmProjectRepository]
        """
        return self._repositories

    @repositories.setter
    def repositories(self, repositories):
        """Sets the repositories of this OrmProjectEdges.


        :param repositories: The repositories of this OrmProjectEdges.  # noqa: E501
        :type repositories: list[OrmProjectRepository]
        """

        self._repositories = repositories

    @property
    def run_executions(self):
        """Gets the run_executions of this OrmProjectEdges.  # noqa: E501


        :return: The run_executions of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmRunExecution]
        """
        return self._run_executions

    @run_executions.setter
    def run_executions(self, run_executions):
        """Sets the run_executions of this OrmProjectEdges.


        :param run_executions: The run_executions of this OrmProjectEdges.  # noqa: E501
        :type run_executions: list[OrmRunExecution]
        """

        self._run_executions = run_executions

    @property
    def run_specs(self):
        """Gets the run_specs of this OrmProjectEdges.  # noqa: E501


        :return: The run_specs of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmRunSpec]
        """
        return self._run_specs

    @run_specs.setter
    def run_specs(self, run_specs):
        """Sets the run_specs of this OrmProjectEdges.


        :param run_specs: The run_specs of this OrmProjectEdges.  # noqa: E501
        :type run_specs: list[OrmRunSpec]
        """

        self._run_specs = run_specs

    @property
    def service_account_subjects(self):
        """Gets the service_account_subjects of this OrmProjectEdges.  # noqa: E501


        :return: The service_account_subjects of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmSubject]
        """
        return self._service_account_subjects

    @service_account_subjects.setter
    def service_account_subjects(self, service_account_subjects):
        """Sets the service_account_subjects of this OrmProjectEdges.


        :param service_account_subjects: The service_account_subjects of this OrmProjectEdges.  # noqa: E501
        :type service_account_subjects: list[OrmSubject]
        """

        self._service_account_subjects = service_account_subjects

    @property
    def tag_group(self):
        """Gets the tag_group of this OrmProjectEdges.  # noqa: E501


        :return: The tag_group of this OrmProjectEdges.  # noqa: E501
        :rtype: OrmTagGroup
        """
        return self._tag_group

    @tag_group.setter
    def tag_group(self, tag_group):
        """Sets the tag_group of this OrmProjectEdges.


        :param tag_group: The tag_group of this OrmProjectEdges.  # noqa: E501
        :type tag_group: OrmTagGroup
        """

        self._tag_group = tag_group

    @property
    def volume(self):
        """Gets the volume of this OrmProjectEdges.  # noqa: E501


        :return: The volume of this OrmProjectEdges.  # noqa: E501
        :rtype: OrmVolume
        """
        return self._volume

    @volume.setter
    def volume(self, volume):
        """Sets the volume of this OrmProjectEdges.


        :param volume: The volume of this OrmProjectEdges.  # noqa: E501
        :type volume: OrmVolume
        """

        self._volume = volume

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmProjectEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmProjectEdges):
            return True

        return self.to_dict() != other.to_dict()



import os
from typing import List, Tuple

import boto3
import botocore.client
from boto3.s3.transfer import S3Transfer
from botocore.credentials import Credentials, RefreshableCredentials
from botocore.session import Session as BotocoreSession

from vessl import vessl_api
from vessl.cli._util import sizeof_fmt
from vessl.util.common import remove_prefix
from vessl.util.downloader import Downloader
from vessl.util.exception import (
    InvalidParamsError,
    InvalidVolumeFileError,
    VesslApiException,
)


class FileTransmission:
    def __init__(self, source_path, source_abs_path, dest_abs_path, size):
        self.source_path = source_path
        self.source_abs_path = source_abs_path
        self.dest_abs_path = dest_abs_path
        self.size = size


class VolumeFileTransfer:
    def __init__(self, volume_id: int):
        self.volume = vessl_api.volume_read_api(volume_id=volume_id)
        self.provider = None
        self.region_name = None
        self.bucket_name = None
        self.prefix = None
        self.endpoint = None
        self.s3_client = None
        self._update_federation_credentials()

    def download(self, source_path, dest_path):
        if self.provider == "gs":
            result = vessl_api.volume_file_list_api(
                volume_id=self.volume.id,
                recursive=True,
                path=source_path,
                need_download_url=True,
            ).results
            files = sorted(result, key=lambda x: x.path)
            Downloader.download(source_path, dest_path, *files, quiet=True)
            return

        (
            file_transmissions,
            total_size,
        ) = self._get_download_file_transmissions_and_total_size(source_path, dest_path)
        total_file_count = len(file_transmissions)
        if total_file_count == 0:
            print("No files to download.")
            return

        formatted_total_size = sizeof_fmt(total_size)
        print(f"Downloading {total_file_count} file(s) ({formatted_total_size})...")

        succeeded_count = 0
        succeeded_size = 0
        for file_transmission in file_transmissions:
            dirname = os.path.dirname(file_transmission.dest_abs_path)
            if dirname:
                os.makedirs(dirname, exist_ok=True)
            try:
                self._get_s3_client().download_file(
                    self.bucket_name,
                    file_transmission.source_abs_path,
                    file_transmission.dest_abs_path,
                    Callback=self._create_callback(succeeded_size, total_size),
                )
                succeeded_count += 1
                succeeded_size += file_transmission.size
            except BaseException as e:
                print(f"Failed to download {file_transmission.source_path}.")
                raise e

        print(f"Total {succeeded_count} file(s) downloaded.")

    def upload(self, source_path, dest_path):
        if self.volume.is_read_only:
            print("Cannot upload to read-only volume.")
            return

        s3_client = self._get_s3_client()
        (
            file_transmissions,
            total_size,
        ) = self._get_upload_file_transmissions_and_total_size(source_path, dest_path)
        total_file_count = len(file_transmissions)
        if total_file_count == 0:
            print("No files to upload.")
            return

        formatted_total_size = sizeof_fmt(total_size)
        print(f"Uploading {total_file_count} file(s) ({formatted_total_size})...")

        succeeded_count = 0
        succeeded_size = 0
        succeeded_files = []
        for file_transmission in file_transmissions:
            try:
                s3_client.upload_file(
                    file_transmission.source_abs_path,
                    self.bucket_name,
                    file_transmission.dest_abs_path,
                    Callback=self._create_callback(succeeded_size, total_size),
                )
                succeeded_count += 1
                succeeded_size += file_transmission.size
                succeeded_files.append({"path": os.path.basename(file_transmission.dest_abs_path)})
            except BaseException as e:
                print(f"Failed to upload {file_transmission.source_path}.")
                raise e

        print(f"Total {succeeded_count} file(s) uploaded.")
        return succeeded_files

    def copy(self, source_path, dest_path):
        # TODO
        return

    def remove(self, path):
        # TODO
        return

    def _create_callback(self, current_size, total_size):
        if total_size < 100 * 1024 * 1024:
            return None

        total_transmitted = current_size
        last_percent = int(total_transmitted / total_size * 100)
        formatted_size = sizeof_fmt(total_size)
        interval = max(int(31 * 1024 * 1024 / total_size * 100), 3)

        def callback(transmitted_bytes):
            nonlocal total_transmitted, last_percent
            total_transmitted += transmitted_bytes
            percent = int(total_transmitted / total_size * 100)
            if percent - last_percent >= interval or percent == 100:
                print(f"{sizeof_fmt(total_transmitted)}/{formatted_size} ({percent}%) completed.")
                last_percent = percent

        return callback

    def _update_federation_credentials(self):
        federation_credentials = vessl_api.volume_federate_api(volume_id=self.volume.id)
        self.region_name = federation_credentials.region
        self.bucket_name = federation_credentials.bucket
        self.prefix = federation_credentials.prefix
        self.provider = federation_credentials.token.provider
        self.endpoint = federation_credentials.endpoint

    def __get_s3_api_endpoint_url(self):
        return os.environ.get("VESSL_VOLUME_S3_API_ENDPOINT_URL", self.endpoint)

    def __get_s3_api_verify_ssl(self):
        if os.environ.get("VESSL_VOLUME_S3_API_VERIFY_SSL") == "false":
            return False
        return None

    def _get_s3_client(self) -> S3Transfer:
        if self.s3_client:
            return self.s3_client

        no_refresh = os.environ.get("VESSL_VOLUME_CREDENTIAL_NO_REFRESH") == "true"
        if no_refresh:
            return self._get_s3_client_from_static_credentials()

        refreshable_credentials = RefreshableCredentials.create_from_metadata(
            metadata=self.__get_s3_session_credentials(),
            refresh_using=self.__get_s3_session_credentials,
            method="sts-assume-role",
        )

        botocore_session = BotocoreSession()
        botocore_session._credentials = refreshable_credentials
        botocore_session.set_config_variable("region", self.region_name)
        session = boto3.session.Session(botocore_session=botocore_session)
        config = botocore.client.Config(connect_timeout=120, read_timeout=120)
        self.s3_client = session.client(
            "s3",
            config=config,
            endpoint_url=self.__get_s3_api_endpoint_url(),
            verify=self.__get_s3_api_verify_ssl(),
        )
        return self.s3_client

    def _get_s3_client_from_static_credentials(self) -> S3Transfer:
        if self.s3_client:
            return self.s3_client

        metadata = self.__get_s3_session_credentials()
        credential = Credentials(
            access_key=metadata["access_key"], secret_key=metadata["secret_key"]
        )
        botocore_session = BotocoreSession()
        botocore_session._credentials = credential
        botocore_session.set_config_variable("region", self.region_name)
        session = boto3.session.Session(botocore_session=botocore_session)
        config = botocore.client.Config(connect_timeout=120, read_timeout=120)
        self.s3_client = session.client(
            "s3",
            config=config,
            endpoint_url=self.__get_s3_api_endpoint_url(),
            verify=self.__get_s3_api_verify_ssl(),
        )
        return self.s3_client

    def __get_all_s3_objects(self, **base_kwargs):
        continuation_token = None
        while True:
            list_kwargs = dict(MaxKeys=1000, **base_kwargs)
            if continuation_token:
                list_kwargs["ContinuationToken"] = continuation_token

            response = self._get_s3_client().list_objects_v2(**list_kwargs)
            yield from filter(
                lambda x: not x["Key"].endswith("/") or x["Size"] > 0, response.get("Contents", [])
            )

            if not response.get("IsTruncated"):
                break

            continuation_token = response.get("NextContinuationToken")

    def _get_download_file_transmissions_and_total_size(
        self, source_path: str, dest_path: str
    ) -> Tuple[List[FileTransmission], int]:
        source_path = source_path.strip("/")
        contents_only = source_path == "." or source_path.endswith("/.")
        if contents_only:
            source_path = source_path[:-2] if source_path.endswith("/.") else ""
        is_root = source_path == ""
        source_abs_path = (
            os.path.join(self.prefix, source_path).replace(os.sep, "/").rstrip("/")
            if self.prefix
            else source_path
        )
        dest_abs_path = os.path.abspath(dest_path)

        objects = list(self.__get_all_s3_objects(Bucket=self.bucket_name, Prefix=source_abs_path))
        if len(objects) == 0:
            return [], 0

        if len(objects) > 1 and os.path.exists(dest_abs_path) and not os.path.isdir(dest_abs_path):
            raise InvalidParamsError(f"{dest_abs_path} is not a directory.")

        if len(objects) == 1 and source_path == remove_prefix(objects[0]["Key"], self.prefix + "/"):
            file_transmission = FileTransmission(
                source_path=source_path,
                source_abs_path=objects[0]["Key"],
                dest_abs_path=dest_abs_path,
                size=objects[0]["Size"],
            )
            if os.path.isdir(dest_abs_path):
                file_transmission.dest_abs_path = os.path.join(
                    dest_abs_path, source_path.split("/")[-1]
                )

            return [file_transmission], file_transmission.size

        prefix = source_abs_path if contents_only else source_abs_path.rsplit("/", 1)[0]
        total_size = 0
        file_transmissions = []
        for obj in objects:
            total_size += obj["Size"]
            if is_root:
                source_rel_path = remove_prefix(obj["Key"], self.prefix + "/")
            else:
                source_rel_path = remove_prefix(obj["Key"], prefix + "/")

            file_transmissions.append(
                FileTransmission(
                    source_path=source_rel_path,
                    source_abs_path=obj["Key"],
                    dest_abs_path=os.path.join(dest_abs_path, source_rel_path),
                    size=obj["Size"],
                )
            )

        return file_transmissions, total_size

    def _get_upload_file_transmissions_and_total_size(
        self, source_path: str, dest_path: str
    ) -> Tuple[List[FileTransmission], int]:
        contents_only = source_path.endswith("/.")
        source_path = os.path.relpath(source_path)
        if not os.path.exists(source_path):
            raise InvalidParamsError(f"{source_path} does not exist.")

        dest_path = dest_path.strip("/")
        dest_abs_path = (
            os.path.join(self.prefix, dest_path).replace(os.sep, "/").rstrip("/")
            if self.prefix
            else dest_path
        )

        s3_client = self._get_s3_client()
        exist = True
        is_root = dest_path == ""
        is_dir = is_root
        try:
            s3_client.head_object(Bucket=self.bucket_name, Key=dest_abs_path)
        except botocore.exceptions.ClientError as e:
            if e.response["Error"]["Code"] == "404":
                response = s3_client.list_objects_v2(Bucket=self.bucket_name, Prefix=dest_abs_path)
                if response["KeyCount"] == 0:
                    exist = False
                else:
                    is_dir = True
            else:
                raise

        source_file_name = os.path.basename(source_path)
        source_abs_path = os.path.abspath(source_path)
        if os.path.isfile(source_path):
            if not exist or not is_dir:
                file_transmission = FileTransmission(
                    source_path=source_path,
                    source_abs_path=source_abs_path,
                    dest_abs_path=dest_abs_path,
                    size=os.path.getsize(source_abs_path),
                )
                return [file_transmission], file_transmission.size
            else:
                file_transmission = FileTransmission(
                    source_path=source_path,
                    source_abs_path=source_abs_path,
                    dest_abs_path=os.path.join(dest_abs_path, source_file_name).replace(
                        os.sep, "/"
                    ),
                    size=os.path.getsize(source_abs_path),
                )
                return [file_transmission], file_transmission.size

        elif not exist or is_dir:
            total_size = 0
            file_transmissions = []
            for root_path, _, file_names in os.walk(source_path):
                for file_name in file_names:
                    source_file_path = os.path.join(root_path, file_name)
                    source_file_abs_path = os.path.abspath(source_file_path)
                    prefix = source_abs_path if contents_only else source_abs_path.rsplit("/", 1)[0]
                    source_rel_path = remove_prefix(source_file_abs_path, prefix + "/")

                    file_size = os.path.getsize(source_abs_path)
                    file_transmissions.append(
                        FileTransmission(
                            source_file_path,
                            source_file_abs_path,
                            os.path.join(dest_abs_path, source_rel_path).replace(os.sep, "/"),
                            file_size,
                        )
                    )
                    total_size += file_size

            return file_transmissions, total_size
        else:
            raise InvalidParamsError(f"{dest_path} is not a directory.")

    def __get_s3_session_credentials(self):
        federation_credentials = vessl_api.volume_federate_api(volume_id=self.volume.id)
        creds = federation_credentials.token.s3
        return {
            "access_key": creds.access_key_id,
            "secret_key": creds.secret_access_key,
            "token": creds.session_token,
            "expiry_time": creds.expiration.isoformat(),
        }



from typing import List

from openapi_client import OrmModelServiceGatewayTrafficSplitEntry as TrafficSplitEntry
from openapi_client import (
    ResponseKernelImage,
    ResponseModelServiceGatewayInfo,
    ResponseModelServiceRevision,
)
from openapi_client.models import OrmAutoscalerConfig, OrmWorkloadPort
from vessl.cli._util import print_data
from vessl.util.exception import InvalidYAMLError


def print_revision(revision: ResponseModelServiceRevision, verbose: bool = False):
    if not verbose:
        data = {
            "Number": revision.number,
            "Status": revision.status,
            "Message": revision.message,
        }
    else:
        kernel_image: ResponseKernelImage = revision.kernel_image
        autoscaler_config: OrmAutoscalerConfig = revision.autoscaler_config
        ports: List[OrmWorkloadPort] = revision.ports.ports
        data = {
            "Number": revision.number,
            "Status": revision.status,
            "Message": revision.message,
            "Available Replicas": revision.available_replicas,
            "Deployment Spec": {
                "Image URL": kernel_image.image_url,
                "Ports": [
                    {"Name": port.name, "Port": port.port, "Type": port.expose_type}
                    for port in ports
                ],
            },
            "Autoscaler Config": {
                "Min Replicas": autoscaler_config.min_replicas,
                "Max Replicas": autoscaler_config.min_replicas,
                "Resource": autoscaler_config.resource_target,
                "Target": autoscaler_config.average_utilization,
            },
        }
    print_data(data)


def print_gateway(gateway: ResponseModelServiceGatewayInfo):
    def _prettify_traffic_split_entry(entry: TrafficSplitEntry) -> str:
        """
        Returns a pretty representation of given traffic split entry.

        Examples:
        - "########   80%:   3 (port 8000)"
        - "##         20%:  12 (port 3333)"
        """
        gauge_char = "#"
        gauge_width = (entry.traffic_weight + 9) // 10  # round up (e.g. 31%-40% -> 4)
        gauge = gauge_char * gauge_width

        return (
            f"{gauge: <10} {entry.traffic_weight: <3}%: {entry.revision_number: >3} "
            f"(port {entry.port})"
        )

    print_data(
        {
            "Enabled": gateway.enabled,
            "Status": gateway.status,
            "Endpoint": gateway.endpoint or "(not set)",
            "Ingress Class": gateway.ingress_class or "(empty)",
            "Annotations": gateway.annotations or "(empty)",
            "Traffic Targets": (
                list(map(_prettify_traffic_split_entry, gateway.rules))
                if gateway.rules
                else "(empty)"
            ),
        }
    )


def list_http_ports(yaml_obj) -> List[int]:
    def _check_type(obj, type_: type):
        assert isinstance(obj, type_)
        return obj

    if "ports" not in yaml_obj:
        return []

    http_ports: List[int] = []
    try:
        ports: list = _check_type(yaml_obj["ports"], list)
        for port in ports:
            expose_type = _check_type(port["type"], str)
            if expose_type == "http":
                port_number = _check_type(port["port"], int)
                http_ports.append(port_number)
    except (KeyError, AssertionError) as e:
        raise InvalidYAMLError(message=str(e))

    return http_ports


def validate_port_choice(yaml_obj):
    pass



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmPipelineStepExecutionEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'access_token': 'OrmAccessToken',
        'execution': 'OrmPipelineExecution',
        'judged_user': 'OrmUser',
        'run_execution': 'OrmRunExecution',
        'step_spec': 'OrmPipelineStepSpec',
        'triggering_pipeline_execution': 'OrmPipelineExecution',
        'workload': 'OrmWorkload'
    }

    attribute_map = {
        'access_token': 'access_token',
        'execution': 'execution',
        'judged_user': 'judged_user',
        'run_execution': 'run_execution',
        'step_spec': 'step_spec',
        'triggering_pipeline_execution': 'triggering_pipeline_execution',
        'workload': 'workload'
    }

    def __init__(self, access_token=None, execution=None, judged_user=None, run_execution=None, step_spec=None, triggering_pipeline_execution=None, workload=None, local_vars_configuration=None):  # noqa: E501
        """OrmPipelineStepExecutionEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._access_token = None
        self._execution = None
        self._judged_user = None
        self._run_execution = None
        self._step_spec = None
        self._triggering_pipeline_execution = None
        self._workload = None
        self.discriminator = None

        if access_token is not None:
            self.access_token = access_token
        if execution is not None:
            self.execution = execution
        if judged_user is not None:
            self.judged_user = judged_user
        if run_execution is not None:
            self.run_execution = run_execution
        if step_spec is not None:
            self.step_spec = step_spec
        if triggering_pipeline_execution is not None:
            self.triggering_pipeline_execution = triggering_pipeline_execution
        if workload is not None:
            self.workload = workload

    @property
    def access_token(self):
        """Gets the access_token of this OrmPipelineStepExecutionEdges.  # noqa: E501


        :return: The access_token of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :rtype: OrmAccessToken
        """
        return self._access_token

    @access_token.setter
    def access_token(self, access_token):
        """Sets the access_token of this OrmPipelineStepExecutionEdges.


        :param access_token: The access_token of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :type access_token: OrmAccessToken
        """

        self._access_token = access_token

    @property
    def execution(self):
        """Gets the execution of this OrmPipelineStepExecutionEdges.  # noqa: E501


        :return: The execution of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :rtype: OrmPipelineExecution
        """
        return self._execution

    @execution.setter
    def execution(self, execution):
        """Sets the execution of this OrmPipelineStepExecutionEdges.


        :param execution: The execution of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :type execution: OrmPipelineExecution
        """

        self._execution = execution

    @property
    def judged_user(self):
        """Gets the judged_user of this OrmPipelineStepExecutionEdges.  # noqa: E501


        :return: The judged_user of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :rtype: OrmUser
        """
        return self._judged_user

    @judged_user.setter
    def judged_user(self, judged_user):
        """Sets the judged_user of this OrmPipelineStepExecutionEdges.


        :param judged_user: The judged_user of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :type judged_user: OrmUser
        """

        self._judged_user = judged_user

    @property
    def run_execution(self):
        """Gets the run_execution of this OrmPipelineStepExecutionEdges.  # noqa: E501


        :return: The run_execution of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :rtype: OrmRunExecution
        """
        return self._run_execution

    @run_execution.setter
    def run_execution(self, run_execution):
        """Sets the run_execution of this OrmPipelineStepExecutionEdges.


        :param run_execution: The run_execution of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :type run_execution: OrmRunExecution
        """

        self._run_execution = run_execution

    @property
    def step_spec(self):
        """Gets the step_spec of this OrmPipelineStepExecutionEdges.  # noqa: E501


        :return: The step_spec of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :rtype: OrmPipelineStepSpec
        """
        return self._step_spec

    @step_spec.setter
    def step_spec(self, step_spec):
        """Sets the step_spec of this OrmPipelineStepExecutionEdges.


        :param step_spec: The step_spec of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :type step_spec: OrmPipelineStepSpec
        """

        self._step_spec = step_spec

    @property
    def triggering_pipeline_execution(self):
        """Gets the triggering_pipeline_execution of this OrmPipelineStepExecutionEdges.  # noqa: E501


        :return: The triggering_pipeline_execution of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :rtype: OrmPipelineExecution
        """
        return self._triggering_pipeline_execution

    @triggering_pipeline_execution.setter
    def triggering_pipeline_execution(self, triggering_pipeline_execution):
        """Sets the triggering_pipeline_execution of this OrmPipelineStepExecutionEdges.


        :param triggering_pipeline_execution: The triggering_pipeline_execution of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :type triggering_pipeline_execution: OrmPipelineExecution
        """

        self._triggering_pipeline_execution = triggering_pipeline_execution

    @property
    def workload(self):
        """Gets the workload of this OrmPipelineStepExecutionEdges.  # noqa: E501


        :return: The workload of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :rtype: OrmWorkload
        """
        return self._workload

    @workload.setter
    def workload(self, workload):
        """Sets the workload of this OrmPipelineStepExecutionEdges.


        :param workload: The workload of this OrmPipelineStepExecutionEdges.  # noqa: E501
        :type workload: OrmWorkload
        """

        self._workload = workload

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmPipelineStepExecutionEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmPipelineStepExecutionEdges):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponsePersistentArtifact(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_by': 'ResponseUser',
        'created_dt': 'datetime',
        'full_path': 'str',
        'managed_storage_configuration': 'V1ManagedStorageConfiguration',
        'name': 'str',
        'nfs_storage_configuration': 'V1NFSStorageConfiguration',
        'parent_artifact_id': 'int',
        'path_description': 'str',
        's3_storage_configuration': 'V1S3StorageConfiguration',
        'storage_type': 'str'
    }

    attribute_map = {
        'created_by': 'created_by',
        'created_dt': 'created_dt',
        'full_path': 'full_path',
        'managed_storage_configuration': 'managed_storage_configuration',
        'name': 'name',
        'nfs_storage_configuration': 'nfs_storage_configuration',
        'parent_artifact_id': 'parent_artifact_id',
        'path_description': 'path_description',
        's3_storage_configuration': 's3_storage_configuration',
        'storage_type': 'storage_type'
    }

    def __init__(self, created_by=None, created_dt=None, full_path=None, managed_storage_configuration=None, name=None, nfs_storage_configuration=None, parent_artifact_id=None, path_description=None, s3_storage_configuration=None, storage_type=None, local_vars_configuration=None):  # noqa: E501
        """ResponsePersistentArtifact - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_by = None
        self._created_dt = None
        self._full_path = None
        self._managed_storage_configuration = None
        self._name = None
        self._nfs_storage_configuration = None
        self._parent_artifact_id = None
        self._path_description = None
        self._s3_storage_configuration = None
        self._storage_type = None
        self.discriminator = None

        if created_by is not None:
            self.created_by = created_by
        if created_dt is not None:
            self.created_dt = created_dt
        if full_path is not None:
            self.full_path = full_path
        if managed_storage_configuration is not None:
            self.managed_storage_configuration = managed_storage_configuration
        if name is not None:
            self.name = name
        if nfs_storage_configuration is not None:
            self.nfs_storage_configuration = nfs_storage_configuration
        self.parent_artifact_id = parent_artifact_id
        if path_description is not None:
            self.path_description = path_description
        if s3_storage_configuration is not None:
            self.s3_storage_configuration = s3_storage_configuration
        if storage_type is not None:
            self.storage_type = storage_type

    @property
    def created_by(self):
        """Gets the created_by of this ResponsePersistentArtifact.  # noqa: E501


        :return: The created_by of this ResponsePersistentArtifact.  # noqa: E501
        :rtype: ResponseUser
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this ResponsePersistentArtifact.


        :param created_by: The created_by of this ResponsePersistentArtifact.  # noqa: E501
        :type created_by: ResponseUser
        """

        self._created_by = created_by

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponsePersistentArtifact.  # noqa: E501


        :return: The created_dt of this ResponsePersistentArtifact.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponsePersistentArtifact.


        :param created_dt: The created_dt of this ResponsePersistentArtifact.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def full_path(self):
        """Gets the full_path of this ResponsePersistentArtifact.  # noqa: E501


        :return: The full_path of this ResponsePersistentArtifact.  # noqa: E501
        :rtype: str
        """
        return self._full_path

    @full_path.setter
    def full_path(self, full_path):
        """Sets the full_path of this ResponsePersistentArtifact.


        :param full_path: The full_path of this ResponsePersistentArtifact.  # noqa: E501
        :type full_path: str
        """

        self._full_path = full_path

    @property
    def managed_storage_configuration(self):
        """Gets the managed_storage_configuration of this ResponsePersistentArtifact.  # noqa: E501


        :return: The managed_storage_configuration of this ResponsePersistentArtifact.  # noqa: E501
        :rtype: V1ManagedStorageConfiguration
        """
        return self._managed_storage_configuration

    @managed_storage_configuration.setter
    def managed_storage_configuration(self, managed_storage_configuration):
        """Sets the managed_storage_configuration of this ResponsePersistentArtifact.


        :param managed_storage_configuration: The managed_storage_configuration of this ResponsePersistentArtifact.  # noqa: E501
        :type managed_storage_configuration: V1ManagedStorageConfiguration
        """

        self._managed_storage_configuration = managed_storage_configuration

    @property
    def name(self):
        """Gets the name of this ResponsePersistentArtifact.  # noqa: E501


        :return: The name of this ResponsePersistentArtifact.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ResponsePersistentArtifact.


        :param name: The name of this ResponsePersistentArtifact.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def nfs_storage_configuration(self):
        """Gets the nfs_storage_configuration of this ResponsePersistentArtifact.  # noqa: E501


        :return: The nfs_storage_configuration of this ResponsePersistentArtifact.  # noqa: E501
        :rtype: V1NFSStorageConfiguration
        """
        return self._nfs_storage_configuration

    @nfs_storage_configuration.setter
    def nfs_storage_configuration(self, nfs_storage_configuration):
        """Sets the nfs_storage_configuration of this ResponsePersistentArtifact.


        :param nfs_storage_configuration: The nfs_storage_configuration of this ResponsePersistentArtifact.  # noqa: E501
        :type nfs_storage_configuration: V1NFSStorageConfiguration
        """

        self._nfs_storage_configuration = nfs_storage_configuration

    @property
    def parent_artifact_id(self):
        """Gets the parent_artifact_id of this ResponsePersistentArtifact.  # noqa: E501


        :return: The parent_artifact_id of this ResponsePersistentArtifact.  # noqa: E501
        :rtype: int
        """
        return self._parent_artifact_id

    @parent_artifact_id.setter
    def parent_artifact_id(self, parent_artifact_id):
        """Sets the parent_artifact_id of this ResponsePersistentArtifact.


        :param parent_artifact_id: The parent_artifact_id of this ResponsePersistentArtifact.  # noqa: E501
        :type parent_artifact_id: int
        """

        self._parent_artifact_id = parent_artifact_id

    @property
    def path_description(self):
        """Gets the path_description of this ResponsePersistentArtifact.  # noqa: E501


        :return: The path_description of this ResponsePersistentArtifact.  # noqa: E501
        :rtype: str
        """
        return self._path_description

    @path_description.setter
    def path_description(self, path_description):
        """Sets the path_description of this ResponsePersistentArtifact.


        :param path_description: The path_description of this ResponsePersistentArtifact.  # noqa: E501
        :type path_description: str
        """

        self._path_description = path_description

    @property
    def s3_storage_configuration(self):
        """Gets the s3_storage_configuration of this ResponsePersistentArtifact.  # noqa: E501


        :return: The s3_storage_configuration of this ResponsePersistentArtifact.  # noqa: E501
        :rtype: V1S3StorageConfiguration
        """
        return self._s3_storage_configuration

    @s3_storage_configuration.setter
    def s3_storage_configuration(self, s3_storage_configuration):
        """Sets the s3_storage_configuration of this ResponsePersistentArtifact.


        :param s3_storage_configuration: The s3_storage_configuration of this ResponsePersistentArtifact.  # noqa: E501
        :type s3_storage_configuration: V1S3StorageConfiguration
        """

        self._s3_storage_configuration = s3_storage_configuration

    @property
    def storage_type(self):
        """Gets the storage_type of this ResponsePersistentArtifact.  # noqa: E501


        :return: The storage_type of this ResponsePersistentArtifact.  # noqa: E501
        :rtype: str
        """
        return self._storage_type

    @storage_type.setter
    def storage_type(self, storage_type):
        """Sets the storage_type of this ResponsePersistentArtifact.


        :param storage_type: The storage_type of this ResponsePersistentArtifact.  # noqa: E501
        :type storage_type: str
        """
        allowed_values = ["managed", "s3", "nfs"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and storage_type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `storage_type` ({0}), must be one of {1}"  # noqa: E501
                .format(storage_type, allowed_values)
            )

        self._storage_type = storage_type

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponsePersistentArtifact):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponsePersistentArtifact):
            return True

        return self.to_dict() != other.to_dict()



import os.path
import subprocess
from typing import List

from vessl.util.vesslignore import list_files_using_vesslignore


class Tar:
    @staticmethod
    def gzip(output_file_name: str, path_to_zip: str, exclude_paths: List = ()) -> int:
        """
        exclude_paths: relative path from path_to_zip
        """
        commands = ["tar", "-C", path_to_zip]
        for exclude_path in exclude_paths:
            commands.extend(["--exclude", exclude_path])

        commands.extend(["-zcf", output_file_name, "."])

        subprocess.run(commands).check_returncode()
        return os.path.getsize(output_file_name)

    @staticmethod
    def gzip_using_vesslignore(
        output_file_name: str,
        path_to_zip: str,
        exclude_paths: List = (),
        vesslignore_filename: str = ".vesslignore",
    ) -> int:
        """
        Make a tarball of the directory `path_to_zip`, using vesslignore pattern file.

        The files are recursively listed and filtered by vesslignore logic.
        The resulting list is supplied to tar via standard input (using the command line parameter '-X').
        """
        commands = ["tar", "-c"]
        commands.extend(["-z"])  # gzip
        commands.extend(
            ["--no-recursion"]
        )  # we choose which files to add; do not add all files when given a directory
        for exclude_path in exclude_paths:
            commands.extend(["--exclude", exclude_path])
        commands.extend(["-C", path_to_zip])  # move base directory
        commands.extend(["-f", output_file_name])  # output file
        commands.extend(["-T", "-"])  # take the list of files to include from stdin

        all_files_to_include = list_files_using_vesslignore(
            path_to_zip, vesslignore_filename=vesslignore_filename
        )

        subprocess.run(
            commands, input="\n".join(all_files_to_include), encoding="utf-8"
        ).check_returncode()

        return os.path.getsize(output_file_name)

    @staticmethod
    def extract(tar_file_name: str, path_to_extract: str):
        subprocess.run(["mkdir", "-p", path_to_extract]).check_returncode()
        subprocess.run(["tar", "xf", tar_file_name, "-C", path_to_extract]).check_returncode()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmOnPremiseVolumeConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cluster_id': 'int',
        'flex_volume': 'OrmOnPremiseVolumeConfigFlexVolume',
        'google_disk': 'OrmOnPremiseVolumeConfigGoogleDisk',
        'host_path': 'OrmOnPremiseVolumeConfigHostPath',
        'minio_endpoint': 'str',
        'nfs': 'OrmOnPremiseVolumeConfigNFS',
        'type': 'str'
    }

    attribute_map = {
        'cluster_id': 'cluster_id',
        'flex_volume': 'flex_volume',
        'google_disk': 'google_disk',
        'host_path': 'host_path',
        'minio_endpoint': 'minio_endpoint',
        'nfs': 'nfs',
        'type': 'type'
    }

    def __init__(self, cluster_id=None, flex_volume=None, google_disk=None, host_path=None, minio_endpoint=None, nfs=None, type=None, local_vars_configuration=None):  # noqa: E501
        """OrmOnPremiseVolumeConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cluster_id = None
        self._flex_volume = None
        self._google_disk = None
        self._host_path = None
        self._minio_endpoint = None
        self._nfs = None
        self._type = None
        self.discriminator = None

        self.cluster_id = cluster_id
        if flex_volume is not None:
            self.flex_volume = flex_volume
        if google_disk is not None:
            self.google_disk = google_disk
        if host_path is not None:
            self.host_path = host_path
        self.minio_endpoint = minio_endpoint
        if nfs is not None:
            self.nfs = nfs
        self.type = type

    @property
    def cluster_id(self):
        """Gets the cluster_id of this OrmOnPremiseVolumeConfig.  # noqa: E501


        :return: The cluster_id of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :rtype: int
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this OrmOnPremiseVolumeConfig.


        :param cluster_id: The cluster_id of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :type cluster_id: int
        """
        if self.local_vars_configuration.client_side_validation and cluster_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cluster_id`, must not be `None`")  # noqa: E501

        self._cluster_id = cluster_id

    @property
    def flex_volume(self):
        """Gets the flex_volume of this OrmOnPremiseVolumeConfig.  # noqa: E501


        :return: The flex_volume of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :rtype: OrmOnPremiseVolumeConfigFlexVolume
        """
        return self._flex_volume

    @flex_volume.setter
    def flex_volume(self, flex_volume):
        """Sets the flex_volume of this OrmOnPremiseVolumeConfig.


        :param flex_volume: The flex_volume of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :type flex_volume: OrmOnPremiseVolumeConfigFlexVolume
        """

        self._flex_volume = flex_volume

    @property
    def google_disk(self):
        """Gets the google_disk of this OrmOnPremiseVolumeConfig.  # noqa: E501


        :return: The google_disk of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :rtype: OrmOnPremiseVolumeConfigGoogleDisk
        """
        return self._google_disk

    @google_disk.setter
    def google_disk(self, google_disk):
        """Sets the google_disk of this OrmOnPremiseVolumeConfig.


        :param google_disk: The google_disk of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :type google_disk: OrmOnPremiseVolumeConfigGoogleDisk
        """

        self._google_disk = google_disk

    @property
    def host_path(self):
        """Gets the host_path of this OrmOnPremiseVolumeConfig.  # noqa: E501


        :return: The host_path of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :rtype: OrmOnPremiseVolumeConfigHostPath
        """
        return self._host_path

    @host_path.setter
    def host_path(self, host_path):
        """Sets the host_path of this OrmOnPremiseVolumeConfig.


        :param host_path: The host_path of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :type host_path: OrmOnPremiseVolumeConfigHostPath
        """

        self._host_path = host_path

    @property
    def minio_endpoint(self):
        """Gets the minio_endpoint of this OrmOnPremiseVolumeConfig.  # noqa: E501


        :return: The minio_endpoint of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :rtype: str
        """
        return self._minio_endpoint

    @minio_endpoint.setter
    def minio_endpoint(self, minio_endpoint):
        """Sets the minio_endpoint of this OrmOnPremiseVolumeConfig.


        :param minio_endpoint: The minio_endpoint of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :type minio_endpoint: str
        """

        self._minio_endpoint = minio_endpoint

    @property
    def nfs(self):
        """Gets the nfs of this OrmOnPremiseVolumeConfig.  # noqa: E501


        :return: The nfs of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :rtype: OrmOnPremiseVolumeConfigNFS
        """
        return self._nfs

    @nfs.setter
    def nfs(self, nfs):
        """Sets the nfs of this OrmOnPremiseVolumeConfig.


        :param nfs: The nfs of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :type nfs: OrmOnPremiseVolumeConfigNFS
        """

        self._nfs = nfs

    @property
    def type(self):
        """Gets the type of this OrmOnPremiseVolumeConfig.  # noqa: E501


        :return: The type of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this OrmOnPremiseVolumeConfig.


        :param type: The type of this OrmOnPremiseVolumeConfig.  # noqa: E501
        :type type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        allowed_values = ["none", "nfs", "flex-volume", "host-path", "google-disk"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmOnPremiseVolumeConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmOnPremiseVolumeConfig):
            return True

        return self.to_dict() != other.to_dict()
